##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-05-12 by Zhewen Song
##############################################################
import os
import numpy as np
import logging
import shutil
from pymatgen.core.sites import PeriodicSite
from pymatgen.core.structure import Structure
from pymatgen.core.structure import Lattice
from pymatgen.core.structure_modifier import StructureEditor
from pymatgen.util.coord_utils import find_in_coord_list
from pymatgen.io.vaspio import Poscar
from MAST.utility import InputOptions
from MAST.utility import MASTObj
from MAST.utility import MASTError
from MAST.utility import Metadata
from MAST.utility import MASTFile
from MAST.utility import dirutil
from MAST.ingredients.baseingredient import BaseIngredient
from MAST.ingredients.pmgextend.structure_extensions import StructureExtensions
from MAST.ingredients.checker import VaspNEBChecker
from MAST.ingredients.checker import VaspChecker

class ChopIngredient(BaseIngredient):
    def __init__(self, **kwargs):
        allowed_keys = {
            'name' : (str, str(), 'Name of directory'),
            'program': (str, str(), 'Program, e.g. "vasp"'),
            'program_keys': (dict, dict(), 'Dictionary of program keywords'),
            'structure': (Structure, None, 'Pymatgen Structure object')
            }
        BaseIngredient.__init__(self, allowed_keys, **kwargs)

    def _fullpath_childname(self, childname):
        """Get full path of the child directory.
            Args: 
                childname <str>: child directory
        """
        cshort = os.path.basename(childname)
        recipedir = os.path.dirname(self.keywords['name'])
        tryrecipe = os.path.basename(recipedir)
        if not (dirutil.dir_is_in_scratch(tryrecipe)):
            recipedir = os.path.dirname(recipedir)
            tryrecipe = os.path.basename(recipedir)
            if not (dirutil.dir_is_in_scratch(tryrecipe)):
                raise MASTError(self.__class__.__name__, "Could not find child directory %s in %s" % (childname, recipedir))
        return os.path.join(recipedir, cshort)

    def copy_file_with_prepend(self, copyfrom="", copyto="", childdir="", softlink=0):
        """Duplicate an ingredient file into the child
            directory, with the ingredient
            name prepended
            e.g. "OSZICAR" becomes "defect_vac1_q=p2_stat_OSZICAR"
            Args:
                copyfrom <str>: File name to copy, e.g. OSZICAR
                copyto <str>: File name to copy to, e.g. OSZICAR
                childdir <str>: Child directory
                softlink <int>: 0 - copy (default)
                                1 - softlink
        """
        ingname = os.path.basename(self.keywords['name'])
        toname = "%s_%s" % (ingname, copyto)
        self.copy_file(copyfrom, toname, childdir, softlink)

    def copy_file(self, copyfrom="", copyto="", childdir="", softlink=0):
        """Copy a file.
            Args:
                copyfrom <str>: name to copy from, e.g. CONTCAR
                copyto <str>: name to copy to, e.g. POSCAR
                childdir <str>: child directory.
                    If not given, use the same directory.
                softlink <int>: 0 (default) - copy
                                1 - softlink
            Log an error if the file is already found
                and do not copy over.
            Do not copy over if the file is empty.
        """
        mydir = self.keywords['name']
        if copyfrom == "":
            raise MASTError(self.__class__.__name__, "No copy-from file given, ingredient %s" % mydir)
        if copyto == "":
            copyto = copyfrom
        if childdir == "":
            childdir = mydir
        childdir = self._fullpath_childname(childdir)
        if not os.path.isdir(childdir):
            raise MASTError(self.__class__.__name__, "No directory for copying into, at %s" % childdir)
        if childdir == mydir and copyfrom == copyto:
            raise MASTError(self.__class__.__name__, "Copy-from and copy-to are the same file and directory for ingredient %s" % mydir)
        self.logger.info("Attempting to copy %s/%s to %s/%s" % (mydir, copyfrom, childdir, copyto))
        frompath = "%s/%s" % (mydir, copyfrom)
        topath = "%s/%s" % (childdir, copyto)
        if not os.path.isfile(frompath):
            self.logger.error("No file at %s. Skipping copy." % frompath)
        else:
            if os.path.isfile(topath):
                self.logger.error("File found at %s already. Skipping copy." % topath)
            else:
                myfile = MASTFile(frompath)
                if len(myfile.data) == 0:
                    self.logger.error("File at %s is empty. Skipping copy." % frompath)
                else:
                    if softlink == 1:
                        curpath = os.getcwd()
                        os.chdir(childdir)
                        mylink=subprocess.Popen("ln -s %s/%s %s" % (mydir, copyfrom, copyto), shell=True)
                        mylink.wait()
                        os.chdir(curpath)
                        self.logger.info("Softlinked file from %s to %s" % (frompath, topath))
                    else:
                        shutil.copy(frompath, topath)
                        self.logger.info("Copied file from %s to %s" % (frompath, topath))
        return

    def get_saddle_dir(self):
        """Get the saddle directory of an NEB calculation.
            VASP only supported.
        """
        if not 'vasp' in self.program:
            raise MASTError(self.__class__.__name__, "Program %s not supported" % self.program)
        myname = self.keywords['name']
        subdirs = dirutil.immediate_subdirs(myname)
        highenergystr="Not evaluated"
        highenergy=-1000000000000.0
        for subdir in subdirs:
            fullsub = os.path.join(myname, subdir)
            singlechecker = VaspChecker(name=fullsub, program_keys = dict(self.checker.keywords['program_keys']), structure = self.checker.keywords['structure'].copy())
            singlechecker.keywords['program_keys']['mast_program'] = "vasp"
            myenergy = singlechecker.get_energy_from_energy_file()
            if myenergy > highenergy:
                highenergystr = subdir
                highenergy = myenergy
        self.logger.info("Saddle directory: %s" % highenergystr)
        return highenergystr

    def copy_saddle_file(self, oldfname, newfname, childname=""):
        """Forward a file from the saddle of a NEB calculation.
            VASP calculations only.
            Args:
                oldfname <str>: Old file name
                newfname <str>: New file name
                childname <str>: Child directory
        """
        saddledir = self.get_saddle_dir()
        self.copy_file("%s/%s" % (saddledir, oldfname), newfname, childname)
        return
    def copy_saddle_file_with_prepend(self, oldfname, newfname, childname=""):
        """Forward a file from the saddle of a NEB calculation with
            the original NEB prepend, but not the inage name.
            VASP calculations only.
            Args:
                oldfname <str>: Old file name
                newfname <str>: New file name
                childname <str>: Child directory
            e.g. 
            self.copy_saddle_file_with_prepend("CONTCAR","POSCAR", childdir)
            neb_vac1-vac2_stat/03/CONTCAR will be copied into childdir as:
            neb_vac1-vac2_stat_POSCAR
        """
        saddledir = self.get_saddle_dir()
        ingname = os.path.basename(self.keywords['name'])
        prependedname = "%s_%s" % (ingname, newfname)
        self.copy_file("%s/%s" % (saddledir, oldfname), prependedname, childname)
        return
    
    def softlink_file(self, linkfrom="", linkto="", childdir=""):
        """Softlink a file.
            Args:
                linkfrom <str>: name to link from, e.g. CONTCAR
                linkto <str>: name to link to, e.g. POSCAR
                childdir <str>: child directory.
            Log an error if the file is already found
                and do not copy over.
        """
        return self.copy_file(linkfrom, linkto, childdir, 1)

    def file_exists(self, filename=""):
        """Check for the existence of a file in the ingredient
            folder.
            Args:
                filename <str>: File name (e.g. OUTCAR)
            Returns:
                True if the file exists; False otherwise
        """
        checkfile = os.path.join(self.keywords['name'], filename)
        filefound = os.path.isfile(checkfile)
        if not filefound:
            self.logger.warning("File %s not found." % checkfile)
        return filefound

    def file_has_string(self, filename="", searchstring="", last_x_lines=0):
        """Check for a string in the file.
            Args:
                filename <str>: File name (e.g. OUTCAR)
                searchstring <str>: Search string
                last_x_lines <str, will be converted to int>:
                    Optional: search only the last X lines.
                    0 (default) - search all lines. 
            Returns:
                True if the string is found; False otherwise
        """
        if not (self.file_exists(filename)):
            return False
        if searchstring == "":
            self.logger.error("No search string given. Returning None.")
            return None
        checkfile = os.path.join(self.keywords['name'],filename)
        tempopen = MASTFile(checkfile)
        mymatch = tempopen.get_last_x_lines_line_match(searchstring, last_x_lines)
        if mymatch == None:
            return False
        else:
            return True

    def copy_fullpath_file(self, copyfromfullpath="", copyto="", childdir="", softlink=0):
        """Copy a file.
            Args:
                copyfromfullpath <str>: full path of the file
                    to copy from, e.g.
                    //home/user/POSCAR_allstart
                copyto <str>: name to copy to, e.g. POSCAR
                childdir <str>: child directory.
                    If not given, use the ingredient directory.
                softlink <int>: 0 (default) - copy
                                1 - softlink
            Log an error if the file is already found
                and do not copy over.
            Do not copy over if the file is empty.
        """
        mydir = self.keywords['name']
        if copyfromfullpath == "":
            raise MASTError(self.__class__.__name__, "No copy-from file given, ingredient %s" % mydir)
        if copyto == "":
            copyto = os.path.basename(copyfromfullpath)
        if childdir == "":
            childdir = mydir
        childdir = self._fullpath_childname(childdir)
        if not os.path.isdir(childdir):
            raise MASTError(self.__class__.__name__, "No directory for copying into, at %s" % childdir)
        self.logger.info("Attempting to copy %s to %s/%s" % (copyfromfullpath, childdir, copyto))
        topath = "%s/%s" % (childdir, copyto)
        if not os.path.isfile(copyfromfullpath):
            self.logger.error("No file at %s. Skipping copy." % copyfromfullpath)
        else:
            if os.path.isfile(topath):
                self.logger.error("File found at %s already. Skipping copy." % topath)
            else:
                myfile = MASTFile(copyfromfullpath)
                if len(myfile.data) == 0:
                    self.logger.error("File at %s is empty. Skipping copy." % copyfromfullpath)
                else:
                    if softlink == 1:
                        curpath = os.getcwd()
                        os.chdir(childdir)
                        mylink=subprocess.Popen("ln -s %s %s" % (copyfromfullpath, copyto), shell=True)
                        mylink.wait()
                        os.chdir(curpath)
                        self.logger.info("Softlinked file from %s to %s" % (copyfromfullpath, topath))
                    else:
                        shutil.copy(copyfromfullpath, topath)
                        self.logger.info("Copied file from %s to %s" % (copyfromfullpath, topath))
        return

    def write_ingred_input_file(self, fname="", allowed_file="all", upperkey=1, delimiter=" "):
        """Write an input file.
            Args: 
                fname <str>: File name for the ingredient input 
                            file to be created, e.g. INCAR
                allowed_file <str>: File name for the list
                    of allowed keywords. Use "all" to allow
                    all non-mast keywords.
                upperkey <str or int>: 
                        1 - uppercase keywords (default)
                        0 - leave keywords their own case
                delimiter <str>: Delimiter to place between
                    keywords and values in the input file.
                    Default is space.
                    Omit this parameter to use a space.
                    Neither semicolon nor comma may be used
                    as a delimiter, as these are special
                    delimiters in the MAST input file already.
        """
        if delimiter == "":
            delimiter = " "
        if allowed_file.lower() == "all":
            okay_keys = self._get_allowed_non_mast_keywords("", upperkey)
        else:
            okay_keys = self._get_allowed_non_mast_keywords(allowed_file, upperkey)
        my_input = MASTFile()
        for key, value in okay_keys.iteritems():
            my_input.data.append(str(key) + delimiter + str(value) + "\n")
        inputpath = os.path.join(self.keywords['name'],fname)
        if os.path.isfile(inputpath):
            self.logger.error("File already exists at %s. Skipping input file writing." % inputpath)
        else:
            my_input.to_file(inputpath)
        return 

    def write_ordered_ingred_input_file(self, fname="", allowed_file="all", upperkey=1, delimiter=" "):
        """Write an input file.
            Assumes that keyword dictionary is given as
            ["#.keyword"] = value
            For example ["2.fix"]=["nvt"]
            Creates an input with keywords in order.
            Args: 
                fname <str>: File name for the ingredient input 
                            file to be created, e.g. INCAR
                allowed_file <str>: File name for the list
                    of allowed keywords. Use "all" to allow
                    all non-mast keywords.
                upperkey <str or int>: 
                        1 - uppercase keywords (default)
                        0 - leave keywords their own case
                delimiter <str>: Delimiter to place between
                    keywords and values in the input file.
                    Default is space.
                    Omit this parameter to use a space.
                    Neither semicolon nor comma may be used
                    as a delimiter, as these are special
                    delimiters in the MAST input file already.
        """
        if delimiter == "":
            delimiter = " "
        if allowed_file.lower() == "all":
            okay_keys = self._get_allowed_non_mast_keywords("", upperkey)
        else:
            okay_keys = self._get_allowed_non_mast_keywords(allowed_file, upperkey)
        keylist=list()
        for key, value in okay_keys.iteritems():
            key = str(key)
            keynum = int(key.split(".")[0])
            keywordval = key.split(".")[1]
            value = str(value)
            keylist.append([keynum, keywordval, value])
        keylist.sort()
        my_input = MASTFile()
        for keytriplet in keylist:
            my_input.data.append(keytriplet[1] + delimiter + keytriplet[2] + "\n")
        inputpath = os.path.join(self.keywords['name'],fname)
        if os.path.isfile(inputpath):
            self.logger.error("File already exists at %s. Skipping input file writing." % inputpath)
        else:
            my_input.to_file(inputpath)
        return 

    def _get_allowed_non_mast_keywords(self, allowed_file="", upperkey=1):
        """Get the non-mast keywords and make a dictionary.
            Args:
                allowed_file <str>: File name containing
                    allowed keywords.
                    If this argument is not entered,
                    then any non-mast keyword will be allowed.
                upperkey <str or int>: 
                        1 - uppercase keywords (default)
                        0 - leave keywords their own case
            Returns:
                my_dict <dict>: Dictionary of keyword and
                    value entries
        """
        my_dict=dict()
        allowed_list = list()
        if not(allowed_file == ""):
            if os.path.isfile(allowed_file):
                allowed_list = self._get_allowed_keywords(allowed_file, upperkey)
            else:
                allowedpath = os.path.join(dirutil.get_mast_control_path(),'programkeys',allowed_file)
                allowed_list = self._get_allowed_keywords(allowedpath, upperkey)
        for key, value in self.keywords['program_keys'].iteritems():
            if not key[0:5] == "mast_":
                if int(upperkey) == 1:
                    keytry = key.upper()
                else:
                    keytry = key
                keyokay = None
                if len(allowed_list) > 0:
                    if keytry in allowed_list:
                        keyokay = True
                    else:
                        keyokay = False
                        self.logger.warning("Ignoring program key %s for INCAR. To allow this keyword, add it to %s" % (keytry, allowedpath))
                else:
                    keyokay = True
                if keyokay:
                    if type(value)==str and value.lower() in ['false','true']:
                        my_dict[keytry]=value.capitalize() #First letter cap
                    else:
                        my_dict[keytry]=value
        return my_dict
    
    def _get_allowed_keywords(self, allowedpath, upperkey=1):
        """Get allowed keywords.
            Args:
                allowedpath <str>: file path for allowed keywords
                upperkey <int or str>: uppercase allowed keywords
        """
        if not os.path.isfile(allowedpath):
            self.logger.error("No file at %s for allowed keywords. Returning empty list." % allowedpath)
            return list()
        allowed = MASTFile(allowedpath)
        allowed_list=list()
        for entry in allowed.data:
            if upperkey == 1:
                allowed_list.append(entry.strip().upper())
            else:
                allowed_list.append(entry.strip())
        return allowed_list

    def no_setup(self):
        """No setup is needed."""
        return
    def no_update(self, childname=""):
        """No updating is needed."""
        return
    
    def run_command(self, commandstr=""):
        """Run a command. Command needs to be of the form
            "<str>.py arg1 arg2 ...." and cannot contain
            the following characters: *, |, &, <, >
        """
        oopslist = ["*","|","&","<",">"]
        for oopchar in oopslist:
            if oopchar in commandstr:
                self.logger.error("Cannot run command specified by command string (see input file) because of bad character.")
                return 
        stringsplit = commandstr.split()
        if not stringsplit[0].strip()[-3:] == ".py":
            self.logger.error("Cannot run command specified by command string (see input file) because of non-python file.")
            return 
        else:
            import subprocess
            myproc = subprocess.Popen(commandstr, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            myresults = myproc.communicate()[0]
            return myresults

    def write_neb(self):
        """Get the parent structures, sort and match atoms, and interpolate.
            Write images to the appropriate folders.
        """
        parentstructures = self.get_parent_structures()
        parentimagestructures = self.get_parent_image_structures()
        image_structures = list()
        if len(parentimagestructures) == 0:
            sxtend = StructureExtensions(struc_work1=parentstructures[0], struc_work2=parentstructures[1],name=self.keywords['name'])
            image_structures = sxtend.do_interpolation(self.keywords['program_keys']['mast_neb_settings']['images'])
        else:
            image_structures.append(parentstructures[0])
            image_structures.extend(parentimagestructures)
            image_structures.append(parentstructures[1])
        if image_structures == None:
            raise MASTError(self.__class__.__name__,"Bad number of images")
        self.checker.keywords['program_keys']['mast_neb_settings']['image_structures']=image_structures
        self.checker.set_up_program_input()
        self.place_parent_energy_files()
        self.write_submit_script()
        return





        
    def get_parent_structures(self):
        """Assume that parents have written files
            named 'parent_structure_<N>'. 
            For VASP these are CONTCAR-type files.
            Returns:
                [struct_init, struct_fin]: pymatgen Structure objects
        """
        header = os.path.join(self.keywords['name'], "parent_structure_")
        mylabel = BaseIngredient.get_my_label(self, "neb_label").split("-")
        pfpath_init = header + mylabel[0]
        pfpath_fin = header + mylabel[1]
        if not os.path.isfile(pfpath_init):
            raise MASTError(self.__class__.__name__,
                "Error: no parent file at" + pfpath_init)
        if not os.path.isfile(pfpath_fin):
            raise MASTError(self.__class__.__name__,
                "Error: no parent file at" + pfpath_fin)
        struct_init = self.checker.get_structure_from_file(pfpath_init)
        struct_fin = self.checker.get_structure_from_file(pfpath_fin)
        base_struct = self.keywords['structure']
        myimages = self.keywords['program_keys']['mast_neb_settings']['images']
        neblines = self.keywords['program_keys']['mast_neb_settings']['lines']
        sxtendi = StructureExtensions(struc_work1 = struct_init, struc_init = base_struct, name=self.keywords['name'])
        sorted_init = sxtendi.sort_structure_and_neb_lines(neblines, '00', myimages) 
        sxtendf = StructureExtensions(struc_work1 = struct_fin, struc_init = base_struct, name=self.keywords['name'])
        sorted_fin = sxtendf.sort_structure_and_neb_lines(neblines, str(myimages + 1).zfill(2), myimages)
        return [sorted_init, sorted_fin]

    def get_parent_image_structures(self):
        """A low-mesh NEB may have written files
            named 'parent_structure_<N-N>_0N'. 
            For VASP these are CONTCAR-type files.
            Returns:
                list of <Structure>: list of pymatgen Structure objects
        """
        header = "parent_structure_"
        numim = self.keywords['program_keys']['mast_neb_settings']['images']
        imct = 1
        imstrs=list()
        while imct <= numim:
            pfpath=""
            for myfile in os.listdir(self.keywords['name']):
                if (header in myfile) and (str(imct).zfill(2) in myfile):
                    pfpath = os.path.join(self.keywords['name'],myfile)
            if pfpath == "":
                pass
            else:
                struct_im = self.checker.get_structure_from_file(pfpath)
                base_struct = self.keywords['structure']
                neblines = self.keywords['program_keys']['mast_neb_settings']['lines']
                sxtend = StructureExtensions(struc_work1 = struct_im, struc_init = base_struct, name=self.keywords['name'])
                sorted_im = sxtend.sort_structure_and_neb_lines(neblines, str(imct).zfill(2), self.keywords['program_keys']['mast_neb_settings']['images']) 
                imstrs.append(sorted_im)
            imct = imct + 1
        if len(imstrs) > 0 and not (len(imstrs) == numim):
            raise MASTError(self.__class__.__name__, "Incomplete number of forwared images found!")
        return imstrs

    def place_parent_energy_files(self):
        """Assume parents have written files parent_energy_<N>.
            Copy these files into the 00 and 0N directories.
        """
        header = os.path.join(self.keywords['name'], "parent_energy_")
        mylabel=BaseIngredient.get_my_label(self, "neb_label").split("-")
        pfpath1= header + mylabel[0]
        pfpath2= header + mylabel[1]
        pffile1=MASTFile(pfpath1)
        pffile2=MASTFile(pfpath2)
        pffile1.to_file(self.checker.get_path_to_write_neb_parent_energy(1)) #MASTFile contains directory locking.
        pffile2.to_file(self.checker.get_path_to_write_neb_parent_energy(2))
        return



    def write_neb_subfolders(self):
        """Write the static runs to each subfolder.
        """
        myname=self.keywords['name']
        mystr=self.keywords['structure']
        numim = int(self.keywords['program_keys']['mast_neb_settings']['images'])
        singlechecker = self.checker
        if self.program == 'vasp':
            nebchecker = VaspNEBChecker(name=self.checker.keywords['name'], program_keys = dict(self.checker.keywords['program_keys']), structure = self.checker.keywords['structure'].copy())
        else:
            raise MASTError(self.__class__.__name__, "NEB checker not supported for program %s") % self.program
        self.checker = nebchecker
        self.write_neb() #Write the POSCAR and OSZICAR files from existing parent-given structures
        self.checker = singlechecker
        for imct in range(1, numim+1):
            subdir = str(imct).zfill(2)
            newname = os.path.join(myname, subdir)
            self.checker.keywords['name']=newname
            self.checker.set_up_program_input()
            self.keywords['name'] = newname
            self.write_submit_script()
        self.keywords['name'] = myname
        self.checker.keywords['name']=myname
        self.checker.keywords['structure']=mystr
        return
    def write_singlerun(self):
        self.checker.set_up_program_input()
        self.write_submit_script()
    def write_singlerun_automesh(self):
        if not self.checker.has_starting_structure_file():
            raise MASTError(self.__class__.__name__,"No starting structure file for ingredient %s; write_singlerun_automesh cannot be used." % self.keywords['name']) 
        if not 'mast_kpoint_density' in self.keywords['program_keys'].keys():
            raise MASTError(self.__class__.__name__,"No mast_kpoint_density ingredient keyword found for ingredient %s; write_singlerun_automesh cannot be used." % self.keywords['name'])
        self.checker.scale_mesh(self.checker.get_initial_structure_from_directory(self.keywords['name']),int(self.keywords['program_keys']['mast_kpoint_density']))
        self.checker.set_up_program_input()
        self.write_submit_script()
    def write_phonon_multiple(self):
        """Write the multiple phonon files, one for each atom and each direction.
        """
        self.checker.set_up_program_input()
        self.write_submit_script()
        mystructure = self.checker.get_initial_structure_from_directory()
        [pcs,pcr,thresh] = self.get_my_phonon_params()
        sxtend = StructureExtensions(struc_work1 = mystructure, name=self.keywords['name'])
        sdarrlist = sxtend.get_multiple_sd_array(pcs, pcr, thresh)
        if sdarrlist == None:
            raise MASTError(self.__class__.__name__, "No phonons to run!")
        sct=1
        myname=self.keywords['name']
        for sdarr in sdarrlist:
            newname = os.path.join(myname,"phon_%s" % str(sct).zfill(2))
            try:
                os.mkdir(newname)
            except OSError:
                pass
            self.checker.keywords['name']=newname
            self.checker.keywords['structure']=mystructure
            self.checker.set_up_program_input()
            self.checker.add_selective_dynamics_to_structure_file(sdarr)
            self.keywords['name'] = newname
            self.write_submit_script()
            self.checker.keywords['name']=myname
            self.checker.softlink_charge_density_file(newname)
            self.checker.softlink_wavefunction_file(newname)
            sct = sct + 1
        self.checker.keywords['name'] = myname
        self.keywords['name']=myname
        

    def write_phonon_single(self):
        """Write the phonon files to a directory.
        """
        self.checker.set_up_program_input()
        self.write_submit_script()
        mystructure = self.checker.get_initial_structure_from_directory()
        [pcs,pcr,thresh] = self.get_my_phonon_params()
        sxtend = StructureExtensions(struc_work1 = mystructure, name=self.keywords['name'])
        sdarr = sxtend.get_sd_array(pcs, pcr,thresh)
        if sdarr == None:
            return
        self.checker.add_selective_dynamics_to_structure_file(sdarr)

    def get_my_phonon_params(self):
        """Get phonon parameters from 
            ['program_keys']['mast_phonon_settings']['phonon_center_site'] and
            ['program_keys']['mast_phonon_settings']['phonon_center_radius'] 
            ['program_keys']['mast_phonon_settings']['threshold'] 
            Returns:
                [phonon_center_site, phonon_center_radius, threshold]
        """
        myphdict = dict(self.keywords['program_keys']['mast_phonon_settings'])
        return [myphdict['phonon_center_site'],myphdict['phonon_center_radius'],myphdict['threshold']]


    def ready_singlerun(self):
        return BaseIngredient.is_ready_to_run(self)
    def ready_structure(self):
        if self.directory_is_locked():
            return False
        return self.checker.has_starting_structure_file()

    def ready_defect(self):
        self.logger.warning("ready_defect is deprecated. use ready_structure instead")
        return self.ready_structure()

    def ready_neb_subfolders(self):
        """Make sure all subfolders are ready to run."""
        myname=self.keywords['name']
        notready=0
        numim = int(self.keywords['program_keys']['mast_neb_settings']['images'])
        for imct in range(0,numim+2):
            subdir = str(imct).zfill(2)
            newname = os.path.join(myname, subdir)
            self.checker.keywords['name']=newname
            if imct == 0 or imct > numim:
                pass
            else:
                if dirutil.directory_is_locked(newname):
                    notready = notready + 1
                if not self.checker.is_ready_to_run():
                    notready = notready + 1
            imct = imct + 1
        self.checker.keywords['name']=myname
        if notready == 0:
            return True
        else:
            return False

    def ready_subfolders(self):
        """Make sure all subfolders are ready to run."""
        myname=self.keywords['name']
        subdirs = dirutil.walkdirs(myname,1,1)
        notready=0
        if len(subdirs) == 0:
            return False
        for subdir in subdirs:
            newname = os.path.join(myname, subdir)
            self.checker.keywords['name']=newname
            if dirutil.directory_is_locked(newname):
                notready = notready + 1
            if not self.checker.is_ready_to_run():
                notready = notready + 1
        self.checker.keywords['name']=myname
        if notready == 0:
            return True
        else:
            return False

    def run_singlerun(self, mode='serial'):
        return BaseIngredient.run(self, mode)
    def run_neb_subfolders(self):
        """Run all image subfolders."""
        myname=self.keywords['name']
        subdirs = dirutil.walkdirs(myname,1,1)
        imct = 0
        numim = int(self.keywords['program_keys']['mast_neb_settings']['images'])
        for subdir in subdirs:
            if imct == 0 or imct > numim:
                pass
            else:
                newname = os.path.join(myname, subdir)
                self.keywords['name']=newname
                BaseIngredient.run(self,"serial")
            imct = imct + 1
        self.keywords['name']=myname
        return
    def run_subfolders(self):
        """Run all subfolders."""
        myname=self.keywords['name']
        phondirs = dirutil.walkdirs(myname,1,1)
        for phondir in phondirs:
            newname = os.path.join(myname, phondir)
            self.keywords['name']=newname
            BaseIngredient.run(self,"serial")
        self.keywords['name']=myname
        return

    def run_strain(self):
        """Strain the lattice.
            Args:
                Looks for mast_strain in input file, for 
                percent strains.
                mast_strain 1.01 1.01 -0.98
            Returns:
                Creates structure file in directory 
        """
        mystructure = self.checker.get_initial_structure_from_directory()
        mystrain = self.keywords['program_keys']['mast_strain']
        sxtend = StructureExtensions(struc_work1 = mystructure, name=self.keywords['name'])
        strained_structure = sxtend.strain_lattice(mystrain)
        self.checker.write_final_structure_file(strained_structure)
        return

    def run_scale(self):
        try:
            base_structure = self.checker.get_initial_structure_from_directory()
        except: #no initial structure
            base_structure = self.keywords['structure'].copy()
            self.logger.warning("No parent structure detected for induce defect ingredient %s. Using initial structure of the recipe." % self.keywords['name'])
        scalingsize = self.metafile.read_data('scaling_size')
        if scalingsize == None: scalingsize = '1 1 1'
        else: scalingsize = ' '.join(scalingsize.split('x'))
        scalextend = StructureExtensions(struc_work1=base_structure, scaling_size=scalingsize, name=self.keywords['name'])
        scaled = scalextend.scale_structure()
        self.checker.write_final_structure_file(scaled)
        return

    def run_defect(self):
        try:
            base_structure = self.checker.get_initial_structure_from_directory()
        except: #no initial structure
            base_structure = self.keywords['structure'].copy()
            self.logger.warning("No parent structure detected for induce defect ingredient %s. Using initial structure of the recipe." % self.keywords['name'])
        scalingsize = self.metafile.read_data('scaling_size')
        if scalingsize == None: scalingsize = '1 1 1'
        else: scalingsize = ' '.join(scalingsize.split('x'))
        defect = self.keywords['program_keys']['mast_defect_settings']
        scaled = base_structure.copy()
        for key in defect:
            if 'subdefect' in key:
                subdefect = defect[key]
                sxtend = StructureExtensions(struc_work1=scaled, scaling_size=scalingsize, name=self.keywords['name'])
                scaled = sxtend.scale_defect(subdefect, defect['coord_type'], defect['threshold'])
            else:
                pass
        self.checker.write_final_structure_file(scaled)
        return

    def run_scale_defect(self):
        try:
            base_structure = self.checker.get_initial_structure_from_directory() 
        except: #no initial structure
            base_structure = self.keywords['structure'].copy()
            self.logger.warning("No parent structure detected for induce defect ingredient %s. Using initial structure of the recipe." % self.keywords['name'])
        scalextend = StructureExtensions(struc_work1=base_structure, name=self.keywords['name'])
        if not 'mast_scale' in self.keywords['program_keys'].keys():
            raise MASTError(self.__class__.__name__,"No mast_scale ingredient keyword for scaling ingredient %s." % self.keywords['name'])
        scaled = scalextend.scale_structure(self.keywords['program_keys']['mast_scale'])
        defect = self.keywords['program_keys']['mast_defect_settings']
        for key in defect:
            if 'subdefect' in key:
                subdefect = defect[key]
                sxtend = StructureExtensions(struc_work1=scaled, struc_work2=base_structure, name=self.keywords['name'])
                scaled = sxtend.scale_defect(subdefect, defect['coord_type'], defect['threshold'])
            else:
                pass
        self.checker.write_final_structure_file(scaled)
        return

    def run_supercell_defect_set(self, parentdir=""):
        """For finite size scaling.
            Given a text file named supercell_list.txt:

            /some/directory/of/parent/ingredient
            ----------------------------------
            ScalingLMN     V_M     Kpoint mesh
            ----------------------------------
            [4, 4, 4] 2.55350475682 [2, 2, 2]
            [3, 4, 3] 3.00247070991 [2, 2, 2]
            [4, 3, 2] 3.05854316622 [2, 2, 4]
            [3, 3, 2] 3.67455399526 [2, 2, 4]
            [2, 2, 2] 5.10700950784 [4, 4, 4]

            There are folders in the parent ingredient
                4x1x1
                3x2x3 etc. within which are a perfect POSCAR
                file and a correspondingly scaled KPOINTS file.
            Copy each folder with POSCAR and KPOINTS.
            Induce the correct defect group for each new folder
                in the current ingredient, scaling by the
                appropriate amount.
            Args:
                parentdir <str>: Parent directory. If not
                        supplied, method will look for
                        supercell_list.txt in the current
                        directory and use the path 
                        specified within the file.
        """
        myname = self.keywords['name']
        supercell_info = MASTFile(os.path.join(myname, "supercell_list.txt"))
        if parentdir == "":
            scaleset_dir = supercell_info.data[0].strip()
        else:
            scaleset_dir = parentdir
        scale_folders = dirutil.immediate_subdirs(scaleset_dir)
        for scale_folder in scale_folders:
            shutil.copytree(os.path.join(scaleset_dir,scale_folder), os.path.join(myname, scale_folder))
        for scale_folder in scale_folders:
            myfolder = os.path.join(myname, scale_folder)
            singlechecker = VaspChecker(name=myfolder,program_keys = dict(self.checker.keywords['program_keys']))
            scaled = singlechecker.get_initial_structure_from_directory() 
            defect = self.keywords['program_keys']['mast_defect_settings']
            scalestring = scale_folder
            for key in defect:
                if 'subdefect' in key:
                    subdefect = defect[key]
                    sxtend = StructureExtensions(struc_work1=scaled, name=self.keywords['name'])
                    scaled = sxtend.scale_defect_by_LMN(scalestring, subdefect, defect['coord_type'], defect['threshold'])
                else:
                    pass
            singlechecker.write_final_structure_file(scaled)
        return True

    def complete_structure(self):
        if self.directory_is_locked():
            return False
        return self.checker.has_ending_structure_file()
    def complete_singlerun(self):
        return BaseIngredient.is_complete(self)
    def complete_neb_subfolders(self):
        """Make sure all subfolders are complete."""
        myname=self.keywords['name']
        subdirs = dirutil.walkdirs(myname,1,1)
        if len(subdirs) == 0:
            return False
        notready=0
        imct = 0
        numim = int(self.keywords['program_keys']['mast_neb_settings']['images'])
        for subdir in subdirs:
            newname = os.path.join(myname, subdir)
            self.keywords['name']=newname
            self.checker.keywords['name']=newname
            self.errhandler.keywords['name']=newname
            if imct == 0 or imct > numim:
                pass
            elif not self.is_complete():
                notready = notready + 1
            imct = imct + 1
        self.keywords['name']=myname
        self.checker.keywords['name']=myname
        self.errhandler.keywords['name']=myname
        if notready == 0:
            return True
        else:
            return False

    def complete_subfolders(self):
        """Make sure all subfolders are complete."""
        myname=self.keywords['name']
        phondirs = dirutil.walkdirs(myname,1,1)
        notready=0
        if len(phondirs) == 0:
            return False
        for phondir in phondirs:
            newname = os.path.join(myname, phondir)
            self.keywords['name']=newname
            self.checker.keywords['name']=newname
            self.errhandler.keywords['name']=newname
            if not self.is_complete():
                notready = notready + 1
        self.keywords['name']=myname
        self.checker.keywords['name']=myname
        self.errhandler.keywords['name']=myname
        if notready == 0:
            return True
        else:
            return False
    def complete_supercell_defect_set(self):
        """Check that a supercell defect set is complete
            by making sure that there is an ending structure
            (CONTCAR for VASP) in every subfolder.
        """
        myname = self.keywords['name']
        mysubdirs = dirutil.immediate_subdirs(myname)
        oklist=list()
        if len(mysubdirs) == 0: #No subdirectories set yet
            return False
        for subdir in mysubdirs:
            self.checker.keywords['name']=os.path.join(myname, subdir)
            if self.checker.has_ending_structure_file():
                oklist.append(True)
            else:
                oklist.append(False)
        self.checker.keywords['name']=myname
        okarr = np.array(oklist)
        if okarr.all():
            return True
        else:
            return False

    def give_structure(self, childname):
        childname = self._fullpath_childname(childname)
        self.checker.forward_final_structure_file(childname)

    def give_neb_structures_to_neb(self, childname):
        """Update to ANOTHER NEB."""
        childname = self._fullpath_childname(childname)
        myct=1
        while myct <= self.keywords['program_keys']['mast_neb_settings']['images']:
            imno = str(myct).zfill(2)
            impath = os.path.join(self.keywords['name'], imno)
            self.checker.keywords['name'] = impath
            self.checker.forward_final_structure_file(childname,"parent_structure_" + BaseIngredient.get_my_label(self, "neb_label") + '_' + imno)
            myct = myct + 1
    
    def give_supercell_subfolder_file(self, oldfname, newfname, childname):
        """Give each CONTCAR to a corresponding scale1 through scale5
            child folder.
            Args:
                oldfname <str>: Old file name
                newfname <str>: New file name
                childname <str>: Child directory name (fullpath)
        """
        myname = self.keywords['name']
        childbase = os.path.basename(childname)
        if not "scale" in childbase:
            raise MASTError(self.__class__.__name__,"Child directory %s needs 'scale' in its name" % childname)
        namesplit = childbase.split("_")
        for nspl in namesplit:
            if "scale" in nspl:
                scalestr = nspl
                break
        scaleidx = int(scalestr.strip()[5:]) - 1 #e.g. scale3 = index 2
        subdirs = dirutil.immediate_subdirs(myname)
        mysubdir = subdirs[scaleidx]
        self.copy_file(os.path.join(mysubdir, oldfname),newfname,childname)
        


    def give_supercell_defect_kpoints(self, childname):
        """Give each KPOINTS to a corresponding scale1 through scale5
            child folder.
        """
        myname = self.keywords['name']
        if not "scale" in childname:
            raise MASTError(self.__class__.__name__,"Child directory %s needs 'scale' in its name" % childname)
        subdirs = dirutil.immediate_subdirs(myname)





    def give_saddle_structure(self, childname):
        """Forward the middle image structure."""
        childname = self._fullpath_childname(childname)
        myname=self.keywords['name']
        subdirs = dirutil.walkdirs(myname,1,1)
        highenergyimct=0
        imct=0
        highenergy=-1000000000000.0
        #numim = int(self.keywords['program_keys']['images'])
        #middleim = int(numim/2)+1 #returns 1 for 1, 2 for 3 im, 3 for 5 im, etc
        subdirs.sort()
        if self.program == 'vasp_neb' or self.program == 'vasp':
            singlechecker = VaspChecker(name=self.checker.keywords['name'],program_keys = dict(self.checker.keywords['program_keys']),structure = self.checker.keywords['structure'].copy())
        else:
            raise MASTError(self.__class__.__name__, "Program %s not supported" % self.program)
        for subdir in subdirs:
            singlechecker.keywords['name'] = subdir
            myenergy = singlechecker.get_energy_from_energy_file()
            if myenergy > highenergy:
                highenergyimct = imct
                highenergy = myenergy
            imct = imct + 1
        imno = str(highenergyimct).zfill(2)
        impath = os.path.join(self.keywords['name'], imno)
        self.checker.keywords['name'] = impath
        self.checker.forward_final_structure_file(childname)
        self.checker.softlink_charge_density_file(childname)
        self.checker.softlink_wavefunction_file(childname)
        return
    def give_phonon_multiple_forces_and_displacements(self,childname):
        self.checker.combine_dynamical_matrix_files(self.keywords['name'])
        self.checker.combine_displacement_files(self.keywords['name'])
        self.give_phonon_single_forces_and_displacements(childname)
    def give_phonon_single_forces_and_displacements(self, childname):
        #Do NOT forward the CONTCAR structure, since the ending CONTCAR contains a displacement in it. Instead, forward the POSCAR
        childname = self._fullpath_childname(childname)
        self.checker.forward_dynamical_matrix_file(childname)
        self.checker.forward_displacement_file(childname)
        self.checker.forward_initial_structure_file(childname, "POSCAR_prePHON")

    def give_structure_and_energy_to_neb(self, childname):
        childname = self._fullpath_childname(childname)
        label = BaseIngredient.get_my_label(self, "defect_label")
        self.checker.forward_final_structure_file(childname,"parent_structure_" + label)
        self.checker.forward_energy_file(childname, "parent_energy_" + label)
    def give_structure_and_restart_files_softlinks(self, childname):
        childname = self._fullpath_childname(childname)
        self.checker.forward_final_structure_file(childname)
        self.checker.softlink_charge_density_file(childname)
        self.checker.softlink_wavefunction_file(childname)
    
    def give_structure_and_restart_files(self, childname):
        self.give_structure_and_restart_files_softlinks(childname)

    def give_structure_and_restart_files_full_copies(self, childname):
        childname = self._fullpath_childname(childname)
        self.checker.forward_final_structure_file(childname)
        self.checker.forward_charge_density_file(childname)
        self.checker.forward_wavefunction_file(childname)
   
    def give_structure_and_charge_density_full_copy(self, childname):
        childname = self._fullpath_childname(childname)
        self.checker.forward_final_structure_file(childname)
        self.checker.forward_charge_density_file(childname)
    
    def give_structure_and_wavefunction_full_copy(self, childname):
        childname = self._fullpath_childname(childname)
        self.checker.forward_final_structure_file(childname)
        self.checker.forward_wavefunction_file(childname)

    def give_structure_and_charge_density_softlink(self, childname):
        childname = self._fullpath_childname(childname)
        self.checker.forward_final_structure_file(childname)
        self.checker.softlink_charge_density_file(childname)
    
    def give_structure_and_wavefunction_softlink(self, childname):
        childname = self._fullpath_childname(childname)
        self.checker.forward_final_structure_file(childname)
        self.checker.softlink_wavefunction_file(childname)



import random
import os
from ase import Atom, Atoms
from MAST.structopt.generate.defect import gen_solid
from MAST.structopt.generate import gen_pop_box, gen_pop_sphere, generate_dumbbells
from MAST.structopt.generate.Individual import Individual

def get_defect_indiv_random(Optimizer):
    """
    Function to generate a structopt Individual class structure with a defect structure.
    Inputs:
        Optimizer = structopt Optimizer class object
    Outputs:
        individ = structopt Individual class object containing defect structure data
    """
    #Initialize Bulk - Generate or load positions of bulk solid
    if not Optimizer.solidbulk:
        if 'Island_Method' not in Optimizer.algorithm_type:
            outfilename = os.path.join(os.path.join(os.getcwd(),Optimizer.filename),'Bulkfile.xyz')
        else:
            from mpi4py import MPI
            rank = MPI.COMM_WORLD.Get_rank()
            outfilename = os.path.join(os.path.join(os.getcwd(),Optimizer.filename+'-rank'+repr(rank)),'Bulkfile.xyz')
        if Optimizer.evalsolid:
            if Optimizer.parallel:
                from MAST.structopt.tools.setup_calculator import setup_calculator
                Optimizer.calc = setup_calculator(Optimizer)
            bulk1, PureBulkEnpa, stro = gen_solid(Optimizer.solidfile,
                Optimizer.solidcell,outfilename,Optimizer.calc,Optimizer.calc_method)
            Optimizer.output.write(stro)
        else:
            bulk1 = gen_solid(Optimizer.solidfile,Optimizer.solidcell,outfilename)
            PureBulkEnpa = 0
        natomsbulk = len(bulk1)
        Optimizer.solidbulk = bulk1.copy()
        Optimizer.purebulkenpa = PureBulkEnpa
        Optimizer.natomsbulk = natomsbulk
    # Identify nearby atoms for region 2 inclusion
    bulk = Optimizer.solidbulk.copy()
    bulkcom = bulk.get_center_of_mass()
    bulk.translate(-bulkcom)
    if Optimizer.sf != 0:
        bulk.append(Atom(position=[0,0,0]))
        nbulk = Atoms(pbc=True, cell=bulk.get_cell())
        nr2 = Atoms(pbc=True, cell=bulk.get_cell())
        for i in range(len(bulk)-1):
            dist = bulk.get_distance(-1,i)
            if dist <= Optimizer.sf:
                nr2.append(bulk[i])
            else:
                nbulk.append(bulk[i])
    else:
        nbulk = bulk.copy()
        nr2 = Atoms(pbc=True, cell=bulk.get_cell())
    #Update atom list with atoms in region 2
    natlist = []
    for sym,c,m,u in Optimizer.atomlist:
        atsym = [atm for atm in nr2 if atm.symbol==sym]
        natlist.append((sym,len(atsym),m,u))
    # Generate random individual and region 2
    if 'sphere' in Optimizer.generate_flag:
        ind = gen_pop_sphere(Optimizer.atomlist,Optimizer.size)
    elif 'dumbbell' in Optimizer.generate_flag:
        ind = Atoms(cell=[Optimizer.size for i in range(3)], pbc=True)
        for sym,c,m,u in Optimizer.atomlist:
            if c > 0:
                dums = generate_dumbbells(c, dumbbellsym=sym, nindiv=1, solid = Optimizer.solidbulk, size=Optimizer.size)[0]
                ind.extend(dums)
    else:
        ind = gen_pop_box(Optimizer.atomlist,Optimizer.size)
    nnr2 = gen_pop_sphere(natlist, Optimizer.sf*2.0)
    nnr2.translate([-Optimizer.sf,-Optimizer.sf,-Optimizer.sf])
    nnr2.set_pbc(True)
    nnr2.set_cell(bulk.get_cell())
    # Initialize class individual with known values
    individ = Individual(ind)
    individ.purebulkenpa = Optimizer.purebulkenpa
    individ.natomsbulk = Optimizer.natomsbulk
    # Combine individual with R2
    icom = ind.get_center_of_mass()
    ind.translate(-icom)
    ind.extend(nnr2)
    ind.set_pbc(True)
    ind.set_cell(bulk.get_cell())
    # Recenter structure
    nbulk.translate(bulkcom)
    ind.translate(bulkcom)
    individ[0] = ind.copy()
    individ.bulki = nbulk.copy()
    individ.bulko = nbulk.copy()
    bulk = nbulk.copy()
    bul = bulk.copy()
    for atm in individ[0]:
        bul.append(atm)
    indices = []
    for sym,c,m,u in Optimizer.atomlist:
        if c < 0:
            if Optimizer.randvacst:
                alist = [one for one in bul if one.symbol==sym]
                count = abs(c)
                while count > 0:
                    indices.append(random.choice(alist).index)
                    count -= 1
            else:
                pos = individ[0][0:Optimizer.natoms].get_center_of_mass()
                count = abs(c)
                bul.append(Atom(position=pos))
                alist = [one for one in bul if one.symbol==sym]
                alistd = [(bul.get_distance(len(bul)-1,one.index),one.index)
                			for one in alist]
                alistd.sort(reverse=True)
                bul.pop()
                while count > 0:
                    idx = alistd.pop()[1]
                    indices.append(idx)
                    count-=1
    if len(indices) !=0:
        nbulklist = [at for at in bul if at.index not in indices and at.index<len(bulk)]
        nalist = [at for at in bul if at.index not in indices and at.index>=len(bulk)]
        bulkn = Atoms(cell=bulk.get_cell(),pbc=True)
        for atm in nbulklist:
            bulkn.append(atm)
        individ.bulki = bulkn.copy()
        individ.bulko = bulkn.copy()
        newind = Atoms()
        for atm in nalist:
            newind.append(atm)
        newind.set_cell(individ[0].get_cell())
        newind.set_pbc(True)									
        individ[0] = newind
    return individ



from MAST.structopt.inp_out.write_xyz import write_xyz

def write_individual(individ, indivfile):
    """Function to write the data of an individual class object to a flat file
    Input:
        individ = Individual class object to be written
        indivfile = String or fileobject for file to be written to 
    Output:
        No output returned.  Information is written to file
    """
    if isinstance(indivfile, str):
        indivfile=open(indivfile, 'a')
    #Write break
    indivfile.write('----------\n')
    #Write structure information
    indivfile.write('Structure information\n')
    write_xyz(indivfile, individ[0])
    indivfile.write('structure cell = {0}\n'.format(get_atom_cell(individ[0])))
    #Write additional information
    indivfile.write('fitness = {0}\n'.format(individ.fitness))
    indivfile.write('index = {0}\n'.format(individ.index))
    indivfile.write('history_index = {0}\n'.format(individ.index))
    indivfile.write('energy = {0}\n'.format(individ.energy))
    indivfile.write('tenergymx = {0}\n'.format(individ.tenergymx))
    indivfile.write('tenergymin = {0}\n'.format(individ.tenergymin))
    indivfile.write('pressure = {0}\n'.format(individ.pressure))
    indivfile.write('volume = {0}\n'.format(individ.volume))
    indivfile.write('force = {0}\n'.format(individ.force))
    indivfile.write('purebulkenpa = {0}\n'.format(individ.purebulkenpa))
    indivfile.write('natomsbulk = {0}\n'.format(individ.natomsbulk))
    indivfile.write('fingerprint = {0}\n'.format(individ.fingerprint))
    indivfile.write('swaplist = {0}\n'.format(individ.swaplist))
    #Write additional structure information
    indivfile.write('bulki\n')
    write_xyz(indivfile, individ.bulki)
    indivfile.write('bulki cell = {0}\n'.format(get_atom_cell(individ.bulki)))
    indivfile.write('bulko\n')
    write_xyz(indivfile, individ.bulko)
    indivfile.write('bulko cell = {0}\n'.format(get_atom_cell(individ.bulko)))
    indivfile.write('box\n')
    write_xyz(indivfile, individ.box)
    indivfile.write('box cell = {0}\n'.format(get_atom_cell(individ.box)))
    indivfile.write('vacancies\n')
    write_xyz(indivfile, individ.vacancies)
    indivfile.write('vacancies cell = {0}\n'.format(get_atom_cell(individ.vacancies)))
    indivfile.write('swaps\n')
    write_xyz(indivfile, individ.swaps)
    indivfile.write('swaps cell = {0}\n'.format(get_atom_cell(individ.swaps)))
    indivfile.write('Finish')
    indivfile.close()
    return

def get_atom_cell(atomsobj):
    cell = atomsobj.get_cell()
    cell_list = []
    for i in range(3):
        clist = []
        for j in range(3):
            clist.append(cell[i][j])
        cell_list.append(clist)
    return cell_list


from ase import Atom, Atoms

def find_top_layer(surf,topthick):
    """Development function for identifying viable surface atoms.
    *** needs development ***
    """
    top=Atoms()
    bulk=Atoms()
    zs=[z for x,y,z in surf.get_positions()]
    thick=max(zs)-min(zs)
    for one in surf:
        x,y,z=one.position
        if z>=max(zs)-topthick:
            top.append(one)
        else:
            bulk.append(one)
    return top, bulk




##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
from pymatgen.io.vaspio import Poscar
from pymatgen.io.vaspio import Outcar
from pymatgen.io.vaspio import Potcar
from pymatgen.io.vaspio import Incar
from pymatgen.io.vaspio import Kpoints
from pymatgen.core.structure import Structure
from MAST.utility import dirutil
from MAST.utility.mastfile import MASTFile
from MAST.utility import MASTError
from MAST.ingredients.checker import BaseChecker
from MAST.ingredients.checker import VaspChecker
import os
import logging
import pymatgen
import numpy as np
import time
import shutil
class GenericChecker(BaseChecker):
    """Generic checker functions
        The generic program should accept an input file named "input.txt"
        with separate lines of the format:
            keyword <delimiter> value
        The delimiter should be set in mast_input_delimiter:
            mast_input_delimiter =
            The default is mast_input_delimiter None, which corresponds to a space.
        The generic program should be run as:
            <mast_exec value>
            for example, "python myscript.py input.txt"
        The generic starting signal should be given by the presence of a file:
            mast_started_file <filename>
        The generic completion signal should be given in the ingredient
        subsection in the $ingredients section of the input file as:
            mast_complete_file <filename>
            mast_complete_search <string to search for, for completion>
                              OR, use the value None in order to check
                              just the existence of the file.
                              Defaults to None.
    """
    def __init__(self, **kwargs):
        allowed_keys = {
            'name' : (str, str(), 'Name of directory'),
            'program_keys': (dict, dict(), 'Dictionary of program keywords'),
            'structure': (Structure, None, 'Pymatgen Structure object')
            }
        BaseChecker.__init__(self, allowed_keys, **kwargs)

    def is_complete(self):
        """Check for the existence of mast_complete_file (default), or
            for a certain string, given by mast_complete_search"""
        if not 'mast_complete_file' in self.keywords['program_keys'].keys():
            raise MASTError(self.__class__.__name__, "No completion file indicated by mast_complete_file keyword for %s. Cannot determine whether run is complete." % self.keywords['name'])
            return False
        checkfile = os.path.join(self.keywords['name'],self.keywords['program_keys']['mast_complete_file'])
        searchstr = None
        if 'mast_complete_search' in self.keywords['program_keys'].keys():
            searchstr = self.keywords['program_keys']['mast_complete_search']
        if searchstr == "None":
            searchstr = None
        if os.path.isfile(checkfile):
            if searchstr == None:
                return True
            else:
                tempopen = MASTFile(checkfile)
                mymatch = tempopen.get_last_line_match(searchstr)
                if mymatch == None:
                    return False
                else:
                    return True
        else:
            return False

    def is_ready_to_run(self):
        """Generic program is ready to run if the input.txt file
            has been written, and if any files in 
            mast_copy_files are present.
            The mast_copy_files keyword must be a space-delimited list of full file paths.
        """
        dirname = self.keywords['name']
        notready=0
        if not(os.path.isfile(dirname + "/input.txt")):
            notready = notready + 1
        if 'mast_copy_files' in self.keywords['program_keys'].keys():
            myfiles = self.keywords['program_keys']['mast_copy_files'].split()
            for myfile in myfiles:
                fname = os.path.basename(myfile)
                if not os.path.isfile(os.path.join(dirname, fname)):
                    notready = notready + 1
        if notready > 0:
            return False
        else:
            return True

    def _copy_over_any_files(self):
        """
            The mast_copy_files ingredient keyword must be a 
            space-delimited list of full file paths:
            mast_copy_files //home/user/f1 //home/user/f2
        """
        dirname = self.keywords['name']
        if 'mast_copy_files' in self.keywords['program_keys'].keys():
            myfiles = self.keywords['program_keys']['mast_copy_files'].split()
            for myfile in myfiles:
                fname = os.path.basename(myfile)
                if not os.path.isfile(os.path.join(dirname, fname)):
                    shutil.copy(myfile, os.path.join(dirname, fname))

    def _phon_poscar_setup(self):
        """Set up a PHON POSCAR file. Strip out the "elements" line (that is,
            use VASP version 4 format. Also strip out anything beneath the atoms
            line.
        """
        name = self.keywords['name']
        pospath = os.path.join(name, "POSCAR")
        prepath = os.path.join(name, "POSCAR_prePHON")
        if os.path.isfile(pospath): #Already done. Return.
            return
        my_poscar = Poscar.from_file(prepath) 
        my_poscar.selective_dynamics=None #unset SD if it is set
        my_poscar.velocities=None #unset velocities
        dirutil.lock_directory(name)
        my_poscar.write_file(pospath)
        dirutil.unlock_directory(name)
        #pick up a copy and strip out the elements line.
        mypfile = MASTFile(pospath)
        myline6=mypfile.get_line_number(6)
        if myline6.strip().split()[0].isalpha:
            mypfile.modify_file_by_line_number(6,"D")
        mypfile.to_file(pospath)
        return



    def _input_get_non_mast_keywords(self):
        """Sort out the non-mast keywords and make a dictionary."""
        input_dict=dict()
        #allowedpath = os.path.join(dirutil.get_mast_install_path(), 'MAST',
        #                'ingredients','programkeys','phon_allowed_keywords.py')
        #allowed_list = self._phon_inphon_get_allowed_keywords(allowedpath)
        for key, value in self.keywords['program_keys'].iteritems():
            if not key[0:5] == "mast_":
                input_dict[key]=value
        return input_dict

    def _input_setup(self):
        """Set up the input.txt file. Note that this is not the recipe
            input file, it is the ingredient input file.
        """
        name=self.keywords['name']
        myd = dict()
        myd = self._input_get_non_mast_keywords()
        my_input = MASTFile()
        if not 'mast_delimiter' in self.keywords['program_keys'].keys():
            delim = " "
        else:
            delim = self.keywords['program_keys']['mast_delimiter']
            if delim == "None":
                delim = " "
        for key, value in myd.iteritems():
            my_input.data.append(str(key) + delim + str(value) + "\n")
        my_input.to_file(name + "/input.txt")
        self._copy_over_any_files()
        return 

    def set_up_program_input(self):
        """For the generic program, MAST will only set up the input.txt
            file. All other necessary files must be previously set up and
            located in the input.txt file, for example:
                 StartingFile = ~/structurebank/startingstructure.xyz 
            in input.txt. This file will not be created by MAST.
        """
        self._input_setup()
        return
    def is_started(self):
        """See if the ingredient has been started on
            the queue. For the generic program, a signal file must
            be specified in the ingredients keywords as mast_started_file
        """
        if not 'mast_started_file' in self.keywords['program_keys'].keys():
            raise MASTError(self.__class__.__name__, "No file indicated by mast_started_file keyword for %s. Cannot determine whether run has started." % self.keywords['name'])
            return False
        checkfile = os.path.join(self.keywords['name'],self.keywords['program_keys']['mast_started_file'])
        if os.path.isfile(checkfile):
            return True
        else:
            return False



def calc_dist(atom1,atom2):
    """Function to calculates the x, y, z, and total distance between atoms
    Inputs:
    	atom1 = ASE class atom object
    	atom2 = ASE class atom object
    Outputs:
    	[d,x,y,z] = [total distance, x-direction distance, y-direction distance, z-direction distance]
    	*Note: all distances are in Angstroms
    """
    pos1=atom1.position
    pos2=atom2.position
    x=abs(pos1[0]-pos2[0])
    y=abs(pos1[1]-pos2[1])
    z=abs(pos1[2]-pos2[2])
    d=(x**2+y**2+z**2)**0.5
    return [d,x,y,z]



import copy
import logging

def check_atomlist_concentration(atomlist,natoms, loggername=None):
    """Checks format of atomlist with natoms to ensure number and
    structure of atoms is correct"""
    flag = False
    checkal = sum([abs(c) for ind,c,m,u in atomlist])
    natoms = int(natoms)
    if loggername:
        #logger = initialize_logger(loggername)
        logger = logging.getLogger(loggername)
        logger.info('Check_atomlist_concentration recieved atomlist={0} and natoms={1}'.format(
            atomlist, natoms))
    if checkal < 1:
        if loggername:
            logger.critical('Number of atoms in atomlist cannot be less than 1.')
            logger.debug('Number of atoms is : {0}'.format(checkal))
        raise RuntimeError('Error: In Atom List, Concentration <1')
        # There cannot be less than one atom in the system
        catmlist = None
    elif checkal == 1:
        if natoms != 1:
            if loggername:
                logger.info('Assuming atom list concentration given as fraction of total atoms')
            natmlist = [[atomlist[i][0],int(round(natoms*atomlist[i][1])),atomlist[i][2],atomlist[i][3]] for i in range(len(atomlist))]
            flag = True
        else:
            catmlist = copy.copy(atomlist)
    elif checkal == 100:
        if natms != 100:
            if loggername:
                logger.info('Assuming atom list concentrations give as percent of total atoms')
            natmlist=[(atomlist[i][0],int(round(natoms*atomlist[i][1]/100)),atomlist[i][2],atomlist[i][3]) for i in range(len(atomlist))]
            flag = True
        else:
            catmlist = copy.copy(atomlist)
    else:
        catmlist = copy.copy(atomlist)
    if flag:
        atms=sum([c for ind,c,m,u in natmlist])
        extraatms=natoms-atms
        while extraatms > 0:
            random.choice(natmlist)[1]+=1
            atms=sum([c for ind,c,m,u in natmlist])
            extraatms=natoms-atms
        while extraatms < 0:
            random.choice(natmlist)[1]-=1
            atms=sum([c for ind,c,m,u in natmlist])
            extraatms=natoms-atms
        catmlist = [tuple(one) for one in natmlist]
    return catmlist


from ase import Atoms, Atom
from MAST.structopt.inp_out.write_xyz import write_xyz

def write_pop(Optimizer,pop):
    """
    Function to write the data from a given population.
    Inputs:
        Optimizer = structopt Optimizer class object with output file data
        pop = List of structopt Individual class objects with data to be written
    Outputs:
        None. Data is written to output files provided by Optimizer class object
    """
    Optimizer.output.write('\n--New Population--\n')
    #Write structures
    for ind in pop:
        update_outfile(ind, Optimizer.output)
        if Optimizer.genealogy:
            update_genealogy(ind, Optimizer.Genealogyfile)
        if Optimizer.swaplist: 
            swaplist_check(ind,Optimizer.structure,Optimizer.output)
        if Optimizer.indiv_defect_write:
            write_xyz(Optimizer.ifiles[ind.index],ind[0],ind.energy)
        update_structsumfile(ind, Optimizer.files[Optimizer.nindiv])
        positions = update_structfile(ind, Optimizer.files[ind.index], Optimizer)
        Optimizer.output.write('Number of positions = {0}\n'.format(len(positions)))
    if Optimizer.genealogy:
        Optimizer.Genealogyfile.write('\n')
    return

def update_outfile(ind, outfile):
    outfile.write('Individual {0}\n'.format(ind.index))
    outfile.write(repr(ind[0])+'\n')
    outfile.write('    Genealogy = {0}\n'.format(ind.history_index))
    outfile.write('    Energy = {0}\n'.format(ind.energy))
    outfile.write('    Fitness = {0}\n'.format(ind.fitness))
    outfile.write('    Swaplist = {0}\n'.format(ind.swaplist))

def update_structsumfile(ind, structsumfile):
    structsumfile.write(' Index = {0}\n'.format(ind.index))
    structsumfile.write('    Energy = {0}\n'.format(ind.energy))
    structsumfile.write('    Fitness = {0}\n'.format(ind.fitness))
    structsumfile.write('    Cell = {0}\n'.format(ind[0].get_cell()))
    structsumfile.write('    Pressure = {0}\n'.format(ind.pressure))
    structsumfile.write('    Genealogy = {0}\n'.format(ind.history_index))
    structsumfile.write('    Swaplist = {0}\n'.format(ind.swaplist))
    
def update_structfile(ind, structfile, Optimizer):
    if Optimizer.structure == 'Defect' or Optimizer.structure == 'Surface':
        sols = Atoms()
        sols.extend(ind[0])
        sols.extend(ind.bulki)
    elif Optimizer.structure == 'Crystal':
        sols = ind[0].repeat((3,3,3))
    else:
        sols = ind[0].copy()
    positions = sols.get_positions()
    if Optimizer.vacancy_output:
        for one in ind.vacancies:
            sols.append(Atom(symbol='X',position=one.position))
    Optimizer.output.write('Number of positions = {0}\n'.format(len(positions)))
    write_xyz(structfile, sols, ind.energy)
    return positions

def update_genealogy(ind, genefile):
    genefile.write('{0} '.format(ind.history_index))

def swaplist_check(ind, structype, outfile):
    sanchn = [[sym,0] for sym,c,m,u in Optimizer.atomlist]
    if structype=='Defect':
        solid = ind[0].copy()
        solid.extend(ind.bulki)
    else:
        solid = ind[0].copy()
    for i in range(len(sanchn)):
        nc = len([atm for atm in solid if atm.symbol==sanchn[i][0]])
        nc += [c for sym,c in ind.swaplist if sym==sanchn[i][0]][0]
        sanchn[i][1]=nc
        outfile.write('Atom structure {0} = {1}\n'.format(sanchn[i][0],
            sanchn[i][1]))


##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# This file is obsolete. 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
import os

import numpy as np
import pymatgen as pmg

from MAST.utility import InputOptions
from MAST.utility import MASTObj
from MAST.utility import MASTError
from MAST.utility import MAST2Structure
from MAST.utility.mastfile import MASTFile

ALLOWED_KEYS = {\
                 'input_options'    : (InputOptions, None, 'Input file name'),\
                 'filename'         : (str, "input.py", 'Script file name'),\
               }

class InputPythonCreator(MASTObj):
    """Creates a runnable python file from a *.inp file
    """
    def __init__(self, **kwargs):
        MASTObj.__init__(self, ALLOWED_KEYS, **kwargs)
    
    def write_script(self, dirname="", fname=""):
        """Write the python input script, created from the *.inp input file.
            Args:
                dirname <str>: directory in which to write
                fname <str>: filename to write
            Returns:
                filename <str>: Full file name of the input script created.
        """
        mylines = self.print_input_options()
        if dirname == "":
            dirname = os.getcwd()
        if fname == "":
            fname = 'input.py'
        filename = os.path.join(dirname, fname)
        inputpy = MASTFile()
        inputpy.data = mylines
        inputpy.to_file(filename)
        return filename


    def addnewlines(self, linestoadd):
        """Appends newline to each string in a list
            Args:
                linestoadd <list of str>: lines to add newline to.
        """
        newline = "\n"
        newlist=list()
        for myline in linestoadd:
            try:
                myline = myline + newline
            except:
                raise MASTError(self.__class__.__name__,"Error with " + str(myline))
            newlist.append(myline)
        return newlist

    def print_input_options(self):
        inputoptions = self.keywords['input_options']
        pln=list()
        pln.append("from MAST.utility import InputOptions")
        pln.append("import numpy as np")
        pln.append("#MAST INPUT OPTIONS")
        pln.append("inputoptions = InputOptions()")

        for sectionname,secvalue in inputoptions.options.iteritems():
            pln.append(" ")
            pln.append("###################################")
            pln.append("#Section: %3s" % sectionname)
            pln.append("####################################")
            pln.append(" ")
            pln.extend(inputoptions.print_python_section(sectionname))
            pln.append("inputoptions.options['"+sectionname+"'] = " + sectionname)
        pln.extend(self.print_create_structure_section())
        pln.extend(self.print_mast_command_section())
        return self.addnewlines(pln)

    def print_create_structure_section(self):
        """Print the structure creation section.
            This is a little complex and may not belong here.

            Returns:
                pln <list of str>: list of lines to print out
        """
        pln=list()
        pln.append(" ")
        pln.append("###############################################")
        pln.append("#Structure Creation Section")
        pln.append("################################################")
        pln.append(" ")
        iops = self.keywords['input_options']
        strposfile = iops.get_item('structure','posfile')
        if strposfile is None:
            pln.append("iopscoords=inputoptions.get_item('structure','coordinates')")
            pln.append("iopslatt=inputoptions.get_item('structure','lattice')")
            pln.append("iopsatoms=inputoptions.get_item('structure','atom_list')")
            pln.append("iopsctype=inputoptions.get_item('structure','coord_type')")
            pln.append("from MAST.utility import MAST2Structure")
            pln.append("structure = MAST2Structure(lattice=iopslatt,")
            pln.append("    coordinates=iopscoords, atom_list=iopsatoms,")
            pln.append("    coord_type=iopsctype)")
        elif ('poscar' in strposfile.lower()):
            pln.append("from pymatgen.io.vaspio import Poscar")
            pln.append("structure = Poscar.from_file('" +strposfile+ "').structure")
        elif ('cif' in strposfile.lower()):
            #from pymatgen.io.cifio import CifParser
            #structure = CifParser(strposfile).get_structures()[0]
            pln.append("from pymatgen.io.cifio import CifParser")
            pln.append("structure = CifParser('"+strposfile+"').get_structures()[0]")
        else:
            error = 'Cannot build structure from file %s' % strposfile
            raise MASTError(self.__class__.__name__, error)
        pln.append("inputoptions.update_item('structure','structure',structure)")
        return pln

    def print_mast_command_section(self):
        """Print the mast commands section.
            Returns:
                pln <list of str>: list of lines to append
        """
        pln=list()
        pln.append(" ")
        pln.append("###############################################")
        pln.append("#MAST Command Section")
        pln.append("################################################")
        pln.append(" ")
        inputoptions = self.keywords['input_options']
        recipe_base_name = os.path.basename(inputoptions.options['recipe']['recipe_file'])
        recipe_base_name = recipe_base_name.split('.')[0]
        import time
        timestamp = time.strftime('%Y%m%d%H%M%S')
        self.name = recipe_base_name + '_' + timestamp + '.py'
        
        pln.append("from MAST.mast import MAST")
        pln.append("mast_obj = MAST()")
        pln.append("mast_obj.start_from_input_options(inputoptions)")
        return pln




##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
import sys, os

import pymatgen as pmg
from pymatgen.io.vaspio.vasp_output import Vasprun, Outcar, Oszicar
from pymatgen.io.smartio import read_structure
from MAST.utility import MASTError
from MAST.utility import MASTFile
from MAST.utility.defect_formation_energy.defectformationenergy import DefectFormationEnergy
from MAST.utility import loggerutils
from MAST.utility import fileutil
import logging
from MAST.ingredients.checker import VaspChecker
#from MAST.utility import PickleManager
from MAST.utility.defect_formation_energy.potential_alignment import PotentialAlignment
from MAST.utility.defect_formation_energy.gapplot import GapPlot
from MAST.utility import Metadata
from MAST.parsers import InputParser

class DefectFormationEnergyIngredient(DefectFormationEnergy):
    """Class for calculating the defect formation energy for a completed MAST
        run, as an ingredient whose input file contains
        dfe_label=perfect_label defected_label
        bandgap_lda_or_gga=<float>
        bandgap_hse_or_expt=<float>
        Attributes:
            self.ingdir <str>: Ingredient directory (should be the
                                current directory)
            self.recdir <str>: Recipe directory (one level up from self.ingdir)
            self.plot_threshold <float>: Plotting threshold
            self.input_options <InputOptions>: Input options of recipe
            self.dirs <dict of str>: Dictionary of perfect and defected
                                     directories, as:
                self.dirs['label1']['perfect'] = <str, directory>
                self.dirs['label1']['defected'] = <str, directory>
            self.bandgap_lda_or_gga <float>: LDA or GGA bandgap (usually
                                             underestimated)
            self.bandgap_hse_or_expt <float>: HSE or experimental bandgap
            self.e_defects <dict of list of tuples>: 
                Defect formation energy dictionary, by charge
                self.e_defects[conditions][label] =
                    [(charge <float>, dfe <float>),
                     (charge <float>, dfe <float), etc.]
            self.logger <logging logger>
    """

    def __init__(self, inputtxt="", plot_threshold=0.01):
        """Initialize defect formation energy ingredient
            Args:
                inputtxt <str>: Input text file, containing:
                    dfe_label1=perfect_label defected_label
                    dfe_label2=perfect_label defected_label
                    dfe_label3=perfect_label defected_label etc.
                    bandgap_lda_or_gga=<float>
                    bandgap_hse_or_expt=<float>
                plot_threshold <float>: Plotting threshold value
        """
        self.logger = logging.getLogger('mastmon')
        self.logger = loggerutils.add_handler_for_control(self.logger)
        self.ingdir = os.getcwd() #Should be run in current directory
        self.recdir = os.path.dirname(self.ingdir)
        self.plot_threshold = plot_threshold
        ipdir = os.path.join(self.recdir,'input.inp')
        os.chdir(self.recdir) #change to recipe directory
        ipparser = InputParser(inputfile=ipdir)
        self.input_options = ipparser.parse()
        os.chdir(self.ingdir) #change back to ingredient directory
        self.dirs = dict()
        self.bandgap_lda_or_gga = 0.0
        self.bandgap_hse_or_expt = 0.0
        self.read_input_file(inputtxt)
        self.e_defects = dict()
        #pm = PickleManager(self.directory + '/input_options.pickle')
        #self.input_options = pm.load_variable()
        #from MAST.recipe.recipesetup import RecipeSetup
        #self.recipe_setup = RecipeSetup(inputOptions=self.input_options,recipeFile=os.path.join(self.recdir,'personal_recipe.txt'),workingDirectory=self.directory)
        #self.recipe_plan = self.recipe_setup.start()
        #PickleManager(self.directory + '/mast.pickle').load_variable()

        #self.final_ingredients = list()
        #ingredientlist = self.recipe_plan.ingredients.keys()
        #updatekeys = self.recipe_plan.update_methods.keys()
        #for ingredient in ingredientlist:
        #    if not (ingredient in updatekeys): # has no children
        #        self.final_ingredients.append(ingredient)
        #    else:
        #        if len(self.recipe_plan.update_methods[ingredient]) == 0:
        #            self.final_ingredients.append(ingredient)
        #print self.final_ingredients
        
    def read_input_file(self, inputtxt=""):
        """Read the input file.
            Args:
                inputtxt <str>: Input file name (shortname)
            Returns:
                Sets self.dirs with perfect and defected directories
                Sets self.bandgap_lda_or_gga
                Sets self.bandgap_hse_or_expt
        """
        ifpath = os.path.join(self.ingdir, inputtxt)
        ifile = MASTFile(ifpath)
        for dline in ifile.data:
            dline = dline.strip()
            if dline[0] == "#":
                continue
            if len(dline) == 0:
                continue
            dsplit = dline.split("=",1)
            dkey = dsplit[0]
            dval = dsplit[1]
            if dkey[0:4] == "dfe_":
                mylabel = dkey[4:]
                myperfect = dval.split()[0]
                mydefected = dval.split()[1]
                self.dirs[mylabel] = dict()
                self.dirs[mylabel]['perfect']=myperfect
                self.dirs[mylabel]['defected']=mydefected
            elif dkey == "bandgap_lda_or_gga":
                self.bandgap_lda_or_gga = float(dval)
            elif dkey == "bandgap_hse_or_expt":
                self.bandgap_hse_or_expt = float(dval)
            else:
                pass
        return
    def calculate_defect_formation_energies(self):
        """Calculate all defect formation energies"""
        chempot = self.input_options.get_item('chemical_potentials')
        for conditions, potentials in chempot.items():
            print 'Conditions:  %s' % conditions.upper()
            self.e_defects[conditions] = dict()
            for mylabel in self.dirs.keys():
                self.calculate_single_defect_formation_energy(mylabel, conditions, potentials)


    def calculate_single_defect_formation_energy(self, mylabel, conditions, potentials):
        """Calculate one defect formation energy.
            Args:
                mylabel <str>: Defect label
                conditions <str>: Environment condition, e.g. "As-rich"
                potentials <dict of float>: Dictionary of chemical potentials,
                    e.g.:
                    potentials['As']=-6.0383
                    potentials['Ga']=-3.6080
                    potentials['Bi']=-4.5650
            Returns:
                <float>: Defect formation energy
        """
        pdir = self.dirs[mylabel]['perfect']
        ddir = self.dirs[mylabel]['defected']
        #Get perfect data
        perf_meta = Metadata(metafile='%s/%s/metadata.txt' % (self.recdir, pdir))
        e_perf = self.get_total_energy(pdir)
        #e_perf = float(perf_meta.read_data('energy').split(',')[-1]) #Sometimes the energy is listed more than once.
        efermi = self.get_fermi_energy(pdir)
        struct_perf = self.get_structure(pdir)
        perf_species = dict()
        for site in struct_perf:
            if (str(site.specie) not in perf_species):
                perf_species[str(site.specie)] = 1
            else:
                perf_species[str(site.specie)] += 1

        #print 'Perfect composition: ', perf_species

        #Get defected data
        def_meta = Metadata(metafile='%s/%s/metadata.txt' % (self.recdir, ddir))
        #print def_meta
        label = def_meta.read_data('defect_label')
        charge = int(def_meta.read_data('charge'))
        energy = self.get_total_energy(ddir)
        #energy = float(def_meta.read_data('energy').split(',')[-1])
        structure = self.get_structure(ddir)
        if (label not in self.e_defects[conditions]):
            self.e_defects[conditions][label] = list()
        print 'Calculating DFEs for defect %s with charge %3i.' % (label, charge)
        def_species = dict()
        for site in structure.sites:
            if (site.specie not in def_species):
                def_species[site.specie] = 1
            else:
                def_species[site.specie] += 1

        # Find the differences in the number of each atom type
        # between the perfect and the defect
        struct_diff = dict()
        for specie, number in def_species.items():
            try:
                nperf = perf_species[str(specie)]
            except KeyError:
                nperf = 0
            struct_diff[str(specie)] = number - nperf

        # Get the potential alignment correction
        alignment = self.get_potential_alignment(pdir, ddir)

        # Calculate the base DFE energy
        e_def = energy - e_perf # E_defect - E_perf
        print "TTM DEBUG: e_def: ", e_def
        for specie, number in struct_diff.items():
            mu = potentials[str(specie)]
            #print str(specie), mu, number
            e_def -= (number * mu)
            print "TTM DEBUG: number: ", number
            print "TTM DEBUG: mu: ", mu
            print "TTM DEBUG: e_def -= number*mu: ", e_def
        print "TTM DEBUG: charge: ", charge
        print "TTM DEBUG: alignment: ", alignment
        print "TTM DEBUG: efermi: ", efermi
        e_def += charge * (efermi + alignment) # Add in the shift here!
        print "TTM DEBUG: e_def += charge*(efermi + alignment): ", e_def
        #print '%-15s%-5i%12.5f%12.5f%12.5f%12.5f' % (label.split('_')[1], charge, energy, e_perf, efermi, alignment)
        print 'DFE = %f' % e_def
        self.e_defects[conditions][label].append( (charge, e_def) )
        return

    def get_total_energy(self, directory):
        """Returns the total energy from a directory"""

        abspath = '%s/%s/' % (self.recdir, directory)
        mychecker = VaspChecker(name=abspath)
        return mychecker.get_energy_from_energy_file()
        #if ('OSZICAR' in os.listdir(abspath)):
        #    return Oszicar('%s/OSZICAR' % abspath).final_energy

        #elif ('vasprun.xml' in os.listdir(abspath)):
        ## Modified from the PyMatGen Vasprun.final_energy() function to return E_0
        #    return Vasprun('%s/vasprun.xml' % abspath).ionic_steps[-1]["electronic_steps"][-1]["e_0_energy"]

    def get_fermi_energy(self, directory):
        """Returns the Fermi energy from a directory"""
        abspath = '%s/%s/' % (self.recdir, directory)

        if ('OUTCAR' in os.listdir(abspath)):
            grepfermi = fileutil.grepme("%s/OUTCAR" % abspath, "E-fermi")
            lastfermi=grepfermi[-1]
            fermi = float(lastfermi.split()[2])
            print "TTM DEBUG LAST FERMI: ", fermi
            return fermi
            #print "TTM DEBUG: OUTCAR efermi: ", Outcar('%s/OUTCAR' % abspath).efermi
            #return Outcar('%s/OUTCAR' % abspath).efermi
        elif ('vasprun.xml' in os.listdir(abspath)):
            return Vasprun('%s/vasprun.xml' % abspath).efermi

    def get_structure(self, directory):
        """Returns the final structure from an optimization"""

        abspath = '%s/%s/' % (self.recdir, directory)

        if ('CONTCAR' in os.listdir(abspath)):
            return pmg.read_structure('%s/CONTCAR' % abspath)
        elif ('vasprun.xml' in os.listdir(abspath)):
            return Vasprun('%s/vasprun.xml' % abspath).final_structure

    def get_potential_alignment(self, perf_dir, def_dir):
        """Returns the potential alignment correction used in charge defects"""

        abs_path_perf = '%s/%s/' % (self.recdir, perf_dir)
        abs_path_def = '%s/%s/' % (self.recdir, def_dir)

        pa = PotentialAlignment()

        if ('OUTCAR' in os.listdir(abs_path_perf)):
            perfect_info = pa.read_outcar('%s/%s' % (abs_path_perf, 'OUTCAR'))
            defect_info = pa.read_outcar('%s/%s' % (abs_path_def, 'OUTCAR'))
            print "TTM DEBUG: Perfect PA info: ", perfect_info
            print "TTM DEBUG: Defected PA info: ", defect_info
            pa_output = pa.get_potential_alignment(perfect_info, defect_info)
            print "TTM DEBUG: PA output: ", defect_info
            return pa_output

    def get_defect_formation_energies(self):
        if not self.e_defects:
            self.calculate_defect_formation_energies()

        return self.e_defects

    @property
    def defect_formation_energies(self):
        return self.get_defect_formation_energies()

    @property
    def dfe(self):
        return self.get_defect_formation_energies()

    def print_table(self):
        if not self.e_defects:
            self._calculate_defect_formation_energies()

        myfile = MASTFile()
        for conditions, defects in self.e_defects.items():
            myfile.data.append('\n\nDefect formation energies for %s conditions.\n' % conditions.upper())
            myfile.data.append('%-20s | %10s\n' % ('Defect', 'DFE'))
            myfile.data.append('---------------------------------\n')
            for defect, energies in defects.items():
                for energy in energies:
                    myfile.data.append('%-14s(q=%2i) | %8.4f\n' % (defect, energy[0], energy[1]))
                myfile.data.append(str()) # Add a blank line here
            myfile.data.append('---------------------------------\n')
        myfile.to_file(os.path.join(os.getcwd(),"dfe.txt"))
        for line in myfile.data:
            print line.strip()
            self.logger.info(line.strip())





import random

def fuss1(pop, nkeep, Optimizer):
	"""Selection function to employ basic Fixed Uniform Selection Scheme to select and 
	order individuals in a population.
	No constraint on fuss -> population will diverge
	"""
	newpop = []
	fits = [ind.fitness for ind in pop]
	minindex = min(xrange(len(fits)), key=fits.__getitem__)
	minf = min(fits)
	maxf = max(fits)
	indices = []
	while len(newpop) < nkeep:
		pt = random.uniform(minf,maxf)
		distances = [abs(pt-fit) for fit in fits]
		index = min(xrange(len(distances)), key=distances.__getitem__)
		if index not in indices:
			newpop.append(pop[index])
			indices.append(index)
	if minindex not in indices:
		rm = random.choice(indices)
		newpop = [pop[inx] for inx in indices if inx != rm]
		newpop.append(pop[minindex])
	return newpop


import random
import numpy
from ase import Atom, Atoms
from MAST.structopt.tools.find_defects import find_defects
from MAST.structopt.inp_out import write_xyz

def rotct_rand_clus(ind1, ind2, Optimizer):
    """Rotate atoms cut and splice
    Rotates atoms randomly around center of mass and cuts with xy plane
    Maintains number of atoms
    Maintains concentration of atoms
    Returns individuals to standard positions at end (un-rotates)
    """
    if 'CX' in Optimizer.debug:
        debug = True
    else:
        debug = False
    if Optimizer.structure != 'Defect':
        Optimizer.output.write('Rotate-Cut Random Cluster Cx attempted on Non Defect structure. SKIPPING CX.\n')
    else:
        Optimizer.output.write('Rotate-Cut Random Cluster Cx between individual '+repr(ind1.index)+' and individual '+repr(ind2.index)+'\n')

        #Perserve starting conditions of individual
        indi1 = ind1[0].copy()
        indi2 =ind2[0].copy()

        indi1c,indi1b,vacant1,swap1,stro1 = find_defects(indi1,Optimizer.solidbulk,0)
        indi2c,indi2b,vacant2,swap2,stro2 = find_defects(indi2,Optimizer.solidbulk,0)
    
        if len(indi1c) !=0 and len(indi2c) != 0:
            #Translate individuals so COM is at (0,0,0)
            com1 = indi1c.get_center_of_mass()
            indi1c.translate(-1*com1)
            com2 = indi2c.get_center_of_mass()
            indi2c.translate(-1*com2)
            #Select random axis, random angle, and random position and rotate individuals
            cmax = [min(numpy.maximum.reduce(indi1c.get_positions())[i],numpy.maximum.reduce(indi2c.get_positions())[i]) for i in range(3)]
            cmin = [min(numpy.minimum.reduce(indi1c.get_positions())[i],numpy.minimum.reduce(indi2c.get_positions())[i]) for i in range(3)]
            n=0
            while n<10:
                rax = random.choice(['x', '-x','y','-y','z','-z'])
                rang = random.random()*90
                rpos = [random.uniform(cmin[i]*0.8,cmax[i]*0.8) for i in range(3)]
                indi1c.rotate(rax,a=rang,center=rpos,rotate_cell=False)
                #Search for atoms in individual 1 that are above the xy plane
                group1 = Atoms(cell=ind1[0].get_cell(),pbc=ind1[0].get_pbc())
                indices1=[]
                for one in indi1c:
                    if one.position[2] >= 0:
                        group1.append(one)
                        indices1.append(one.index)
                if len(group1) > 2 and len(group1) < len(indi1):
                    break
                else:
                    n+=1
                    indi1c.rotate(rax,a=-1*rang,center=rpos, rotate_cell=False)
            indi2c.rotate(rax,a=rang,center=rpos,rotate_cell=False)
            if debug: 
                print 'Group1 size = ', len(group1)
                print 'Position = ', rpos
                print 'Angle = ', rang
                print 'Axis = ', rax
                print 'Number of tries = ', n
            if len(group1) != 0:
                #Apply concentration forcing if needed
                group2 = Atoms(cell=ind2[0].get_cell(),pbc=ind2[0].get_pbc())
                indices2 = []
                dellist = []
                for one in indi2c:
                    if one.position[2] >= 0:
                        group2.append(one)
                        indices2.append(one.index)
                if Optimizer.forcing=='Concentration':
                    symlist = list(set(indi1c.get_chemical_symbols()))
                    seplist = [[atm for atm in group2 if atm.symbol == sym] for sym in symlist]
                    group2n=Atoms(cell=group2.get_cell(),pbc=group2.get_pbc())
                    indices2n = []
                    dellist = []
                    for one in group1:
                        sym1=one.symbol
                        listpos=[i for i,s in enumerate(symlist) if s==sym1][0]
                        if len(seplist[listpos]) > 0:
                            pos = random.choice(range(len(seplist[listpos])))
                            group2n.append(seplist[listpos][pos])
                            indices2n.append(indices2[seplist[listpos][pos].index])
                            del seplist[listpos][pos]
                        else:
                            dellist.append(one.index)
                    if len(dellist) != 0:
                        dellist.sort(reverse=True)
                        for one in dellist:
                            del group1[one]
                            del indices1[one]
                    indices2=indices2n
                    group2=group2n.copy()
                else:
                    dellist = []
                    while len(group2) < len(group1)-len(dellist):
                        #Too many atoms in group 1
                        dellist.append(random.choice(group1).index)
                    if len(dellist) != 0:
                        dellist.sort(reverse=True)
                        for one in dellist:
                            del group1[one]
                            del indices1[one]
                    dellist = []
                    while len(group1) < len(group2)-len(dellist):
                        #Too many atoms in group 2
                        dellist.append(random.choice(group2).index)
                    if len(dellist) != 0:
                        dellist.sort(reverse=True)
                        for one in dellist:
                            del group2[one]
                            del indices2[one]

                other2 = Atoms(cell=ind2[0].get_cell(),pbc=ind2[0].get_pbc())
                for one in indi2c:
                    if one.index not in indices2:
                        other2.append(one)

                other1 = Atoms(cell=ind1[0].get_cell(),pbc=ind1[0].get_pbc())
                for one in indi1c:
                    if one.index not in indices1:
                        other1.append(one)
                group1.rotate(rax,a=-1*rang,center=rpos, rotate_cell=False)
                group1.translate(com1)
                other1.rotate(rax,a=-1*rang,center=rpos, rotate_cell=False)
                other1.translate(com1)
                group2.rotate(rax,a=-1*rang,center=rpos, rotate_cell=False)
                group2.translate(com2)
                other2.rotate(rax,a=-1*rang,center=rpos, rotate_cell=False)
                other2.translate(com2)
                indi1 = group2.copy()
                indi1.extend(other1)
                indi1.extend(indi1b)
                indi2 = group1.copy()
                indi2.extend(other2)
                indi2.extend(indi2b)

                #DEBUG: Write crossover to file
                if debug: 
                    write_xyz(Optimizer.debugfile, group1,'group1')
                    write_xyz(Optimizer.debugfile, other1,'other1')
                    write_xyz(Optimizer.debugfile, group2,'group2')
                    write_xyz(Optimizer.debugfile, other2,'other2')
                    print 'Length of group1 = ',len(group1),'Length of group2',len(group2)

                #DEBUG: Check structure of atoms exchanged
                for sym,c,m,u in Optimizer.atomlist:
                    nc=len([atm for atm in indi1 if atm.symbol==sym])
                    Optimizer.output.write('CX RANDROTCT: Individual 1 contains '+repr(nc)+' '+repr(sym)+' atoms\n')
                    nc=len([atm for atm in indi2 if atm.symbol==sym])
                    Optimizer.output.write('CX RANDROTCT: Individual 2 contains '+repr(nc)+' '+repr(sym)+' atoms\n')
                if Optimizer.forcing !='Concentration':
                    for i in range(len(Optimizer.atomlist)):
                        atms1=[inds for inds in indi1 if inds.symbol==Optimizer.atomlist[i][0]]
                        atms2=[inds for inds in indi2 if inds.symbol==Optimizer.atomlist[i][0]]
                        if len(atms1)==0:
                            if len(atms2)==0:
                                indi1[random.randint(0,len(indi1)-1)].symbol==Optimizer.atomlist[i][0]
                                indi2[random.randint(0,len(indi2)-1)].symbol==Optimizer.atomlist[i][0]
                            else:
                                indi1.append(atms2[random.randint(0,len(atms2)-1)])
                                indi1.pop(random.randint(0,len(indi1)-2))
                        else:
                            if len(atms2)==0:
                                indi2.append(atms1[random.randint(0,len(atms1)-1)])
                                indi2.pop(random.randint(0,len(indi2)-2))	

            #DEBUG: Check structure and number of atoms in crystal
            if Optimizer.structure=='Defect':
                solid1=Atoms()
                solid1.extend(indi1)
                solid1.extend(ind1.bulki)
                solid2=Atoms()
                solid2.extend(indi2)
                solid2.extend(ind2.bulki)
                for sym,c,m,u in Optimizer.atomlist:
                    nc=len([atm for atm in solid1 if atm.symbol==sym])
                    Optimizer.output.write('CX RANDROTCT: Defect 1 configuration contains '+repr(nc)+' '+repr(sym)+' atoms\n')
                    nc=len([atm for atm in solid2 if atm.symbol==sym])
                    Optimizer.output.write('CX RANDROTCT: Defect 2 configuration contains '+repr(nc)+' '+repr(sym)+' atoms\n')
            if debug: Optimizer.output.flush()
            #pdb.set_trace()
        else:
            Optimizer.output.write('  Warning: failed to identify a cluster:\n')
            Optimizer.output.write('    Individual '+repr(ind1.index)+' length = '+repr(len(indi1c))+'\n')
            Optimizer.output.write('    Individual '+repr(ind2.index)+' length = '+repr(len(indi2c))+'\n')
        ind1[0]=indi1
        ind2[0]=indi2
    return ind1, ind2



##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
import os
import sys
from MAST.utility import Metadata
from MAST.utility import fileutil
from MAST.ingredients.checker import VaspChecker
from MAST.ingredients.checker import VaspNEBChecker
def main(ingname=""):
    """Get the last pressure value from an ingredient.
        Args:
            ingname <str>: Ingredient name, full path
        Returns:
            <string>: "Last pressure (kbar);0.00", ex:
                "Last pressure (kbar);-23.55"
    """
    trymeta = "%s/metadata.txt" % ingname
    frontstr = "Last pressure (kbar);"
    if os.path.isfile(trymeta):
        mymeta = Metadata(metafile=trymeta)
        myprogram = mymeta.read_data("program")
    else:
        myprogram = "None"
    if myprogram in ['vasp','vasp_neb']:
        if os.path.isdir("%s/01" % ingname):
            tryfile = "%s/01/OUTCAR" % ingname
        else:
            tryfile = "%s/OUTCAR" % ingname
        greppress = fileutil.grepme(tryfile, "external pressure")
        #"external pressure = -1.97 kB Pullay stress = 0.00 kB"
        if greppress == []:
            return "%s N/A" % frontstr
        mypress = greppress[-1].strip().split()[3]
        return "%s %s" % (frontstr, mypress)
    else:
        return "%s N/A" % frontstr



from MAST.structopt.inp_out import read_xyz
from MAST.structopt.tools import find_top_layer
from MAST.structopt.switches import moves_switch
from MAST.structopt.generate.Individual import Individual

def get_surface_indiv(Optimizer):
    """
    Function to generate an structopt Individual class object containing as surface structure.
    Inputs:
        Optimizer = structopt Optimizer class
    Outputs:
        individ = structopt Individual class object containing surface structure data
    """
    #Load surface structure
    surfs = read_xyz(Optimizer.surfacefile)
    cells = Optimizer.surfacecell
    surfs.set_cell(cells)
    surf.set_pbc([True,True,False])
    #Find top layer
    top,bulks=find_top_layer(surfs,Optimizer.surftopthick)
    mutopto = Optimizer.mutation_options
    Optimizer.mutation_options = ['lattice_alteration_rdrd']
    topind = top.copy()
    ind = moves_switch(topind,Optimizer)
    Optimizer.mutation_options = mutopto
    individ = Individual(ind)
    individ.bulki = bulks.copy()
    individ.bulko = bulks.copy()
    return individ



from MAST.structopt.tools import get_best

def best(pop, nkeep, Optimizer=None):
    """Selection function to select the best individuals in a population
    Inputs:
        pop = list of Individual class structures
        nkeep = number of Individuals to keep in the population
            must be less than or equal to the number of individuals currently in the population
        Optimizer = dummy placeholder
    Outputs:
        npop = population with desired number of individuals
    """
    if nkeep <= len(pop):
        npop = get_best(pop,nkeep)
    else:
        print 'WARNING: selection.best nkeep > pop. Returning pop'
        npop = pop
    return npop


import random
from MAST.structopt.tools.find_defects import find_defects

def lattice_alteration_small(indiv, Optimizer):
    """Move function to perform small Lattice Alteration of atoms
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    natomsmove = 0
    ratmlocnew = 0
    if Optimizer.structure=='Defect':
        if Optimizer.isolate_mutation:
            indc,indb,vacant,swap,stro = find_defects(indiv[0],Optimizer.solidbulk,0)
            positions = indc.get_positions()
            if len(positions) != 0:
                try:
                    natomsmove=random.randint(1,len(positions))
                except ValueError:
                    natomsmove=1
            else:
                natomsmove = 0
        else:
            positions=indiv[0].get_positions()
            if len(positions) != 0:
                try:
                    natomsmove=random.randint(1,len(positions)/5)
                except ValueError:
                    natomsmove=1
            else:
                natomsmove = 0
    else:
        positions=indiv[0].get_positions()
        if len(positions) != 0:
            try:
                natomsmove=random.randint(1,len(positions)/5)
            except ValueError:
                natomsmove=1
        else:
            natomsmove = 0
    ratmlocnew=[0]*natomsmove
    for i in range(natomsmove):
        ratmloc=random.randint(0,len(positions)-1)
        ratmlocnew[i]=(random.uniform(-1,1), random.uniform(-1,1), random.uniform(-1,1))
        positions[ratmloc]+=ratmlocnew[i]
    if Optimizer.structure=='Defect':
        if Optimizer.isolate_mutation:
            indc.set_positions(positions)
            indiv[0]=indc.copy()
            indiv[0].extend(indb)
        else:
            indiv[0].set_positions(positions)
    else:
        indiv[0].set_positions(positions)
    Optimizer.output.write('Lattice Alteration Small Mutation performed on individual\n')
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    Optimizer.output.write('Number of atoms moved = '+repr(natomsmove)+'\n')
    Optimizer.output.write(repr(ratmlocnew)+'\n')
    Optimizer.output.write(repr(indiv[0])+'\n')
    muttype='LAS'+repr(natomsmove)
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv




##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Glen Jenness
# Last updated: 2013-07-01
##############################################################

import sys, math
import numpy as np


class PotentialAlignment:
#    def __init__(self):
#        self.outcar_perf = outcar_perf
#        self.outcar_def = outcar_def

    def read_outcar(self, outcar):
        """Reads a VASP OUTCAR file, and extracts out some useful information.
            Args:
                outcar:  OUTCAR file name
            Returns:
                elst_pot:  average electrostatic potential for each atomic center
                atom_list:  list of each atom type
        """
        outfile = open(outcar, 'r')

        atom_types = list()
        atom_numbers = list()
        atom_list = list()

        eof = False
        while (not eof):
            line = outfile.readline()
            if (not line):
                eof = True
            elif 'average (electrostatic) potential at core' in line:
                raw_data = list()
                outfile.readline() # Read in Junk line
                outfile.readline() # Read in Junk line

                pot_line = True
                while (pot_line):
                    pot_line = outfile.readline().strip()
                    if (not line):
                        pot_line = False
                    else:
                        raw_data.append(pot_line)
            elif 'ions per type' in line:
                atom_numbers = map(int, line.split('=')[1].split())
            elif 'VRHFIN' in line:
                atom_types.append(line.split()[1][1:-1])
            elif ' E-fermi :' in line:
                efermi = float(line.split()[2])

        outfile.close()
        elst_pot = list()
#        print raw_data
        for data in raw_data:
            data = data.split('-')
            for datum in data[1:-1]:
                elst_pot.append(-float(datum.split()[0]))
            if data[0]:
                elst_pot.append(-float(data[-1]))

        for i in range(len(atom_numbers)):
            for j in range(atom_numbers[i]):
                atom_list.append(atom_types[i])

        return (elst_pot, atom_list, efermi)

    def get_potential_alignment(self, perfect_info, defect_info):
        """Gets the shift.  Assumes all atoms in the perfect cell are symmetry
            equivalent, i.e. for Sc2O3, by symmetry we have two different Sc
            atoms (8b and 24d Wycoff positions) with two different potentials.

            In reality the best way to do this is to match up the symmetry
            positions of the perfect with that in the defected and do the 
            alignment that way.  However, a quick and dirty way is to average
            over each symmetry position, which allows us to account for the 
            different symmetry environments in an effective way without having
            to dig into the underlying unit cell symmetry.
        """

        perf_elst_pot, perf_atom_list, perf_efermi = perfect_info
        def_elst_pot, def_atom_list, def_efermi = defect_info

        unique_perf = list(set(perf_atom_list))
        ave_pot = list()

        for unique in unique_perf:
            nunique = 0
            unique_sum = 0.0
            for i in range(len(perf_atom_list)):
                if (perf_atom_list[i] == unique):
                    unique_sum += perf_elst_pot[i]
                    nunique += 1

            ave_pot.append(unique_sum / nunique)

#        print ave_pot
        shift = 0.0

        for i in range(len(ave_pot)):
            perf_pot = ave_pot[i]
            atom = unique_perf[i]

            for j in range(len(def_elst_pot)):
                if (atom == def_atom_list[j]):
                    shift += (def_elst_pot[j] - perf_pot)

#        print perf_efermi
        return shift / len(def_elst_pot)



import random
import numpy
from ase import Atom, Atoms
from MAST.structopt.tools.find_defects import find_defects
from MAST.structopt.generate import gen_pop_box

def random_replacement(indiv, Optimizer):
    """Move function to replace selection of atoms with randomly generated group
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    if Optimizer.structure=='Defect':
        if Optimizer.isolate_mutation:
            atms,indb,vacant,swap,stro = find_defects(indiv[0],Optimizer.solidbulk,0)
        else:
            atms = indiv[0].copy()
    else:
        atms=indiv[0].copy()
    nat=len(atms)
    if nat != 0:
        #Select number of atoms to replace
        if nat<=1:
            natrep2=1
            natrep1=0
        elif nat<=5:
            natrep2=2
            natrep1=0
        else: 
            natrep1=random.randint(1,nat/2)
            while True:
                natrep2=random.randint(2,nat/2)
                if natrep2 != natrep1:
                    break
        natrep=abs(natrep2 - natrep1)
        #Select random position in cluster
        maxcell = numpy.maximum.reduce(atms.get_positions())
        mincell = numpy.minimum.reduce(atms.get_positions())
        pt=[random.uniform(mincell[0],maxcell[0]),random.uniform(mincell[1],maxcell[1]),random.uniform(mincell[2],maxcell[2])]
        #Get distance of atoms from random point
        atpt=Atom(position=pt)
        atms.append(atpt)
        dist=[]
        for i in range(len(atms)-1):
            dist.append(atms.get_distance(i,len(atms)-1))
        atms.pop()
        dlist=zip(dist,atms)
        dlist=sorted(dlist, key=lambda one: one[0], reverse=True)
        # Select atoms closest to random point
        atmsr=Atoms()
        indexlist=[]
        for i in range(natrep):
            atmsr.append(dlist[i][1])
            indexlist.append(dlist[i][1].index)
        natomlist=[0]*len(Optimizer.atomlist)
        for i in range(len(Optimizer.atomlist)):
            atms1=[inds for inds in atmsr if inds.symbol==Optimizer.atomlist[i][0]]
            natomlist[i]=(Optimizer.atomlist[i][0], len(atms1),Optimizer.atomlist[i][2],Optimizer.atomlist[i][3])
        nsize = max(numpy.maximum.reduce(atmsr.get_positions())-numpy.minimum.reduce(atmsr.get_positions()))
        repcenter = atmsr.get_center_of_mass()
        atmsn = gen_pop_box(natomlist,nsize)
        atmsn.translate(repcenter)
        #Update individual with new atom positions
        for i in range(len(indexlist)):
            index=indexlist[i]
            atms[index].position=atmsn[i].position
        if Optimizer.structure=='Defect':
            if Optimizer.isolate_mutation:
                atms.extend(indb)
        indiv[0]=atms.copy()
    else:
        natrep=0
        pt=0
    Optimizer.output.write('Random Group Replacement Mutation performed on individual\n')
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    Optimizer.output.write('Number of atoms replaced = '+repr(natrep)+'\n')
    Optimizer.output.write('Geometry point = '+repr(pt)+'\n')
    Optimizer.output.write(repr(indiv[0])+'\n')
    muttype='RGR'+repr(natrep)
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv



import os
import math
import numpy
from MAST.structopt.inp_out import read_xyz
from MAST.structopt.tools import calc_dist

def get_lattice_concentration(bulkfile,indivfile):
    """Function to identify the lattice concentration of atoms in a bulk structure compared to a 
    structure with a defect.
    Inputs:
        bulkfile = filename for starting structure with original lattice atoms
        indivfile = filename for structure to compare
    Outputs:
        File: LatticeConcentration.txt in working directory.  Includes summary of concentration
        of each atom type and vacancies
    ** Note: Currently limited to cubic structures **
    """
    # Load Bulk Solid File
    solid=read_xyz(bulkfile)

    # Get lattice sites for bulk

    #Identify nearest neighbor distance
    solid.set_pbc(True)
    distmin=[]
    for i in range(20):
        dist=[]
        for j in range(len(solid)):
            if i !=j:
                d = calc_dist(solid[i],solid[j])
                dist.append(d)
        distmin.append(min(dist))
    nndist=sum([one for one,x,y,z in distmin])/len(distmin)
    nnxd=sum([x for one,x,y,z in distmin])/len(distmin)
    nnyd=sum([y for one,x,y,z in distmin])/len(distmin)
    nnzd=sum([z for one,x,y,z in distmin])/len(distmin)

    solid.translate([nnxd/2.0,nnyd/2.0,nnzd/2.0])

    # Get size of cell for pbc
    cell = numpy.maximum.reduce(solid.get_positions())
    cell += [nnxd/2.0,nnyd/2.0,nnzd/2.0]

    # Initialize boxes
    nx=int(math.ceil(float(cell[0])/float(nnxd)))
    ny=int(math.ceil(float(cell[1])/float(nnyd)))
    nz=int(math.ceil(float(cell[2])/float(nnzd)))
    bxarray0=[[0,[]] for i in range(nx*ny*nz)]

    # Identify which atoms are in which box
    positions=solid.get_positions()
    for i in range(len(solid)):
        box=[math.floor(positions[i][0]/nnxd),math.floor(positions[i][1]/nnyd),math.floor(positions[i][2]/nnzd)]
        bxarray0[int((nx*ny)*(box[2])+nx*(box[1])+box[0])][0]+=1
        bxarray0[int((nx*ny)*(box[2])+nx*(box[1])+box[0])][1]+=[solid[i].symbol]

    #Get types of atoms for bulk and bulk lattice concentration
    nlatsites=len(solid)
    syms =list(set([atm.symbol for atm in solid]))
    nsyms = []
    for one in syms:
        numberofsym=len([atm for atm in solid if atm.symbol==one])
        nsyms.append(float(numberofsym)/float(nlatsites))
    concentbulk=zip(syms,nsyms)

    #Get Lattice sites for individual
    onlatcon=[[-1,concentbulk]]
    n=0
    while True:
        try:
            indiv=read_xyz(indivfile,n)
        except:
            break
        indiv.translate([nnxd/2.0,nnyd/2.0,nnzd/2.0])
        bxarray=[[0,[],[]] for i in range(nx*ny*nz)]
        positions=indiv.get_positions()
    
        # Wrap positions in individual to cell size
        for i in range(len(positions)):
            while positions[i][0] > cell[0]:
                positions[i][0]=positions[i][0]-cell[0]
            while positions[i][1] > cell[1]:
                positions[i][1]=positions[i][1]-cell[1]
            while positions[i][2] > cell[2]:
                positions[i][2]=positions[i][2]-cell[2]
            while positions[i][0] < 0:
                positions[i][0]=positions[i][0]+cell[0]
            while positions[i][1] < 0:
                positions[i][1]=positions[i][1]+cell[1]
            while positions[i][2] < 0:
                positions[i][2]=positions[i][2]+cell[2]
    
        for i in range(len(indiv)):
            box=[math.floor(positions[i][0]/nnxd),math.floor(positions[i][1]/nnyd),math.floor(positions[i][2]/nnzd)]
            bxarray[int((nx*ny)*(box[2])+nx*(box[1])+box[0])][0]+=1
            bxarray[int((nx*ny)*(box[2])+nx*(box[1])+box[0])][1]+=[indiv[i].symbol]
            bxarray[int((nx*ny)*(box[2])+nx*(box[1])+box[0])][2]+=[i]

        # Get on-lattice concentration
        latsyms=[]
        for i in range(len(bxarray0)):
            if bxarray0[i][0]!=0:
                if len(bxarray[i][1])==1:
                    latsyms.extend(bxarray[i][1])
                elif len(bxarray[i][1]) >1:
                    if bxarray[i][1][0] in bxarray[i][1]:
                        latsyms.extend(bxarray0[i][1])
                else:
                    latsyms.append('Vacancy')
        reducedsyms=list(set(latsyms))
        concenti=[]
        for one	in reducedsyms:
            numberofsym=len([atm for atm in latsyms if atm==one])
            concenti.append(float(numberofsym)/float(nlatsites))
        concents=zip(reducedsyms,concenti)
        onlatcon.append([n,concents])
        n+=1

    maxlen=max([len(con) for n,con in onlatcon])
    maxsyms=[con for n,con in onlatcon if len(con)==maxlen]
    symlist=[sym for sym,n in maxsyms[0]]
    output=open('LatticeConcentration.txt','a')
    output.write('Generation ')
    for one in symlist:
        output.write(repr(one)+' ')
    output.write('TotalSites \n')
    for n,con in onlatcon:
        output.write(repr(n)+' ')
        for sym in symlist:
            num=[count for atm,count in con if atm==sym]
            num=sum(num)
            output.write(repr(num)+' ')
        output.write(repr(nlatsites)+'\n')
    output.close()




import random
import os
import copy
import numpy
from MAST.structopt.tools.lammps import LAMMPS

def cell_shape(indiv, Optimizer):
    """Move function to forcefully alter the unit cell shape
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    Optimizer.output.write('Cell Shape change mutation\n')
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    structure=random.choice(Optimizer.cell_shape_options)
    cello=indiv[0].get_cell()
    if structure=='cubic':
        #Set to cubic shape
        an,bn,cn = [numpy.linalg.norm(v) for v in cello]
        a=(an+bn+cn)/3.0
        celln=numpy.array([[a,0,0],[0,a,0],[0,0,a]])
        Optimizer.output.write('Mutating to cubic\n')
        muttype='CSC'
    elif structure=='hexagonal':
        #Set to hexagonal shape
        an,bn,cn = [numpy.linalg.norm(v) for v in cello]
        a=(an+bn)/2.0
        c=cn
        if c<=a:
            c=random.uniform(a+1,10)
        trans=numpy.array([[1,0,0],[-0.5,(3.0**0.5)/2.0,0],[0,0,1]])
        trans[0]=[a*i for i in trans[0]]
        trans[1]=[a*i for i in trans[1]]
        trans[2]=[c*i for i in trans[2]]
        celln=trans
        Optimizer.output.write('Mutating to Hexagonal\n')
        muttype='CSH'
    elif structure=='tetragonal':
        #Set to tetragonal shape
        an,bn,cn = [numpy.linalg.norm(v) for v in cello]
        a=(an+bn)/2.0
        c=cn
        if c==a:
            c=random.uniform(1,10)
        celln=numpy.array([[a,0,0],[0,a,0],[0,0,c]])
        Optimizer.output.write('Mutating to tetragonal\n')
        muttype='CSTe'
    elif structure=='orthorhombic':
        #Set to orthorhombic
        a=random.uniform(2,10)
        b=random.uniform(2,10)
        c=random.uniform(2,10)
        celln=numpy.array([[a,0,0],[0,b,0],[0,0,c]])
        Optimizer.output.write('Mutating to orthorhombic\n')
        muttype='CSO'
    elif structure=='monoclinic':
        #Set to monoclinic
        a,b,c = [numpy.linalg.norm(v) for v in cello]
        if a==b:
            b=random.uniform(1,10)
        trans=numpy.array([(1+random.random())*c, 0, (1+random.random())*c])
        celln=numpy.array([[a,0,0],[0,b,0],[0,0,0]])
        celln[2]=trans
        Optimizer.output.write('Mutating to monoclinic\n')
        muttype='CSM'
    elif structure=='triclinic':
        #Set to triclinic
        a,b,c = [numpy.linalg.norm(v) for v in cello]
        celln=numpy.array([[a,0,0],[(1+random.random())*b,(1+random.random())*b,0],[(1+random.random())*c,0,(1+random.random())*c]])
        Optimizer.output.write('Mutating to triclinic\n')
        muttype='CSTr'
    indiv[0].set_cell(celln)
    #Relax new box shape with Lammps box/reax for cell
    Optimizer.output.write('LAMMPS fix box/relax performed\n')
    cwd1=os.getcwd()
    parameters=copy.deepcopy(Optimizer.calc.parameters)
    try:
        parameters['mass'][len(parameters['mass'])-1] += '\nfix 1 all box/relax iso 0.0 vmax 0.001'
    except KeyError:
        parameters['pair_coeff'][0] += '\nfix 1 all box/relax iso 0.0 vmax 0.001'
    filesL = [ Optimizer.pot_file ]
    if Optimizer.lammps_keep_files:
        calc2 = LAMMPS(parameters=parameters, files=filesL,keep_tmp_files=True, tmp_dir=os.getcwd()+'/'+Optimizer.filename+'/Lammpsrun2/')
    else:
        calc2 = LAMMPS(parameters=parameters, files=filesL)
    atmsdup=indiv[0].copy()
    atmsdup.set_calculator(calc2)
    try:
        OUT=atmsdup.calc.calculate(atmsdup)
        indiv[0].set_cell(OUT['atoms'].get_cell())
        indiv[0].set_positions(OUT['atoms'].get_positions())
        Optimizer.output.write('Energy = '+repr(OUT['thermo'][-1]['pe']/indiv[0].get_number_of_atoms())+'\n')
    except:
        Optimizer.output.write('WARNING: Minimizer quit - Box relaxation unsuccessful\n')
    #pdb.set_trace()
    os.chdir(cwd1)
    calc2.clean()
    Optimizer.output.write(repr(indiv[0].get_cell())+'\n')
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


from MAST.structopt.fingerprinting import get_fingerprint
from MAST.structopt.switches import selection_switch
from MAST.structopt.tools import remove_duplicates
import math

def adapting(pop, Optimizer):
    """Function to provide an adapting fitness function for GA evaluation
    Input:
        pop = population consisting of list of Individual Class objects to be evaluated
        Optimizer = Optimizer class object with fitness parameters
    Output:
        pop = new population updated based on fitness evaluation
    *** needs work ***
    """
    fitlist = [one.fitness for one in pop]
    nfitlist, nindices = remove_duplicates(fitlist, Optimizer.demin)
    STR = ''
    newpop = []
    if len(nfitlist) != len(fitlist):
        STR+='Predator: Removed total of '+repr(len(fitlist)-len(nfitlist))+' from population\n'
    otherlist = []
    for i in range(len(pop)):
        if i not in nindices:
            STR+='Predator: Removed '+repr(pop[i].history_index)+'\n'
            otherlist.append(pop[i])
        else:
            newpop.append(pop[i])
    while len(newpop) < Optimizer.nindiv:
        Optimizer.output.write('Predator: Adding duplicates back')
        indiv = random.choice(otherlist)
        newpop.append(indiv)
        STR+='Predator: Adding mutated duplicates to new pop history='+indiv.history_index+'\n'
    if Optimizer.natural_selection_scheme=='fussf':
        for ind in newpop:
            if ind.fingerprint == 0:
                ind.fingerprint = get_fingerprint(Optimizer,ind,Optimizer.fpbin,Optimizer.fpcutoff)
    if genrep >= Optimizer.reqrep*Optimizer.adaptbegin:
        ofusslim = Optimizer.fusslimit
        nfusslim = ofusslim*math.exp(-Optimizer.adaptmultiplier*float(Optimizer.genrep)/float(Optimizer.reqrep))
        Optimizer.fusslimit = nfusslim
    else:
        ofusslim = Optimizer.fusslimit
    pop = selection_switch(newpop, Optimizer.nindiv,Optimizer.natural_selection_scheme,Optimizer)
    pop = get_best(pop,len(pop))
    Optimizer.fusslimit=ofusslim
    return pop, STR



'''General functions for selection'''
# from best import *
# from cost import *
# from fuss import *
# from fuss1 import *
# from fuss1r import *
# from fuss2 import *
# from fussf import *
# from fussr import *
# from metropolis import *
# from multitournament import *
# from random_pick import *
# from rank import *
# from tournament import *
# from tournament1 import *
# from tournament2 import *



def remove_duplicates(oldlist, tolerance):
    """Function to remove the duplicates in a list.
    ***A duplicate is defined as being less than a given tolerance
    different from the other numbers in the list***
    Inputs:
        oldlist = List of floats/integers possibly containing duplicates
        tolerance = maximum difference between items for them to be considered duplicates
    Outputs:
        newlist = List without duplicates
        newlistindices = List of indices of items in newlist corresponding to positions in old list
    """
    newlist = []
    dups = []
    newlistindices = []
    for i in range(len(oldlist)):
        diffs = []
        #Identify the difference between given item and other items in list
        for j in range(len(oldlist)):
            diffs.append(abs(oldlist[i]- oldlist[j]))
        #Check if any of the differences are less than the tolerance
        flaggedlist = []
        for k in range(len(diffs)):
            if diffs[k] <= tolerance:
                if k !=i:
                    flaggedlist.append(k)
        if len(flaggedlist) != 0:
            dups.append([i,flaggedlist])
        else:
            newlistindices.append(i)
    selects = []
    for one,others in dups:
        if len(selects)==0:
            selects.append(one)
        else:
            flag=True
            for two in others:
                if two in selects:
                    flag=False
            if flag:
                selects.append(one)
    newlistindices.extend(selects)
    newlistindices.sort()
    for one in newlistindices:
        newlist.append(oldlist[one])
    
    return newlist, newlistindices


from MAST.structopt.switches import fitness_switch
from MAST.structopt.tools import setup_calculator
import copy

def quench(indiv, Optimizer):
    """Move function to perform a molecular dynamics NVT quenching of structure
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    olammps_min = Optimizer.lammps_min
    ocalc = Optimizer.calc
    mincomd = '1.0e-8 1.0e-8 1000 100000\nunfix fix_nve\nfix fix_nvt all nvt temp '+repr(Optimizer.quench_max_temp)+' '+repr(Optimizer.quench_min_temp)+' '+repr(Optimizer.quench_step_size)
    mincomd += '\nrun '+repr(Optimizer.quench_n_steps_1)+'\nunfix fix_nvt\nfix fix_nve all nve\nminimize 1.0e-8 1.0e-8 1000 100000'
    mincomd += '\nunfix fix_nve\nfix fix_nvt all nvt temp '+repr(Optimizer.quench_max_temp)+' '+repr(Optimizer.quench_min_temp)+' '+repr(Optimizer.quench_step_size)
    mincomd += '\nrun '+repr(Optimizer.quench_n_steps_2)+'\nunfix fix_nvt\nfix fix_nve all nve\nminimize 1.0e-8 1.0e-8 1000 100000'
    Optimizer.lammps_min = mincomd
    Optimizer.calc = setup_calculator(Optimizer)
    outs = fitness_switch([Optimizer,indiv])
    indiv = outs[0]
    Optimizer.output.write('Quench NVT Mutation performed on individual\n')
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    Optimizer.output.write(outs[1])
    Optimizer.output.write('New fitness : '+repr(indiv.fitness)+'\n')
    muttype='Q'
    Optimizer.lammps_min = olammps_min
    Optimizer.calc = setup_calculator(Optimizer)
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


# utility/__init__.py

import numpy as np
import pymatgen as pmg

from MAST.utility.mastobj import MASTObj
from MAST.utility.masterror import MASTError
from MAST.utility.inputoptions import InputOptions
from MAST.utility.mastfile import MASTFile
from MAST.utility.metadata import Metadata

def MAST2Structure(lattice=None, coordinates=None, atom_list=None, coord_type='fractional'):
    """Helper function for converting input options into a pymatgen Structure object"""
    if (coord_type.lower() == 'fractional'):
        return pmg.Structure(lattice, atom_list, coordinates)
    elif (coord_type.lower() == 'cartesian'):
        return pmg.Structure(lattice, atom_list, coordinates,
                             coords_are_cartesian=True)



#!/usr/bin/env python
##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
# Purpose: Submit a job to the queue. This function is highly platform-specific and may need to be modified.
# Authors: Tam Mayeshiba
# 10/25/12 TTM created
# 12/6/12 TTM added UKY DLX commands
import os
import sys
from MAST.utility import dirutil
import subprocess
import time
from MAST.utility import MASTFile

def direct_shell_command():
    return "vasp"

def queue_submission_command():
    return "qsub"

def queue_status_from_text(jobid, queuetext):
    """
        Returns the queue status from a line with the job ID information
        INPUTS:
            jobid <int> = job ID
            qstr <str> = queue snapshot line containing job ID
        OUTPUTS:
            <str> = queue status 
            'E': Error
            'X': Not found
            'R': Running
            'Q': Queued
    """
    if (queuetext == None): 
        return 'X'
    if len(queuetext) == 0:
        return 'X'
    begin = queuetext.find(str(jobid))
    if begin == -1:
        return 'X'
    queuetext = queuetext[begin:]
    end = queuetext.find('\n')
    if end > -1:
        queuetext = queuetext[:end]
    
    qmat = queuetext.split()
    if len(qmat) < 5:
        return 'E'
    return qmat[4] #TTM 1/17/12 qstat returns differently than qstat -a does

def extract_submitted_jobid(string):
    """
        Extract the job ID from a string returned when the job is submitted
        INPUTS:
            string <str> = string with job ID somewhere in it
        OUTPUTS:
            <int> = job ID as integer
    """
    if string == "":
        return None
    return int(string.split('.')[0])

def queue_snap_command():
    """
        Return the command for producing a queue snapshot.
        INPUTS:
            None
        OUTPUTS:
            Command for producing a queue snapshot
    """
    return "qstat"

def get_approx_job_error_file(jobid):
    """
        Return the approximate job error file name
            Args:
                jobid <int>: Job ID
    """
    if jobid == None:
        return ".e"
    else:
        return ".e%s" % str(jobid)
